from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import ToleranceUnits


class RunHeader(NetWrapperBase):
    """
    The run header.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.RunHeader

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()

    @property
    def first_spectrum(self) -> int:
        """
        Gets or sets the number for the first scan in this stream (usually 1)
        """
        return self._get_wrapped_object_().FirstSpectrum

    @first_spectrum.setter
    def first_spectrum(self, value: int):
        """
        Gets or sets the number for the first scan in this stream (usually 1)
        """
        assert type(value) is int
        self._get_wrapped_object_().FirstSpectrum = value

    @property
    def last_spectrum(self) -> int:
        """
        Gets or sets the number for the last scan in this stream
        """
        return self._get_wrapped_object_().LastSpectrum

    @last_spectrum.setter
    def last_spectrum(self, value: int):
        """
        Gets or sets the number for the last scan in this stream
        """
        assert type(value) is int
        self._get_wrapped_object_().LastSpectrum = value

    @property
    def start_time(self) -> float:
        """
        Gets or sets the time of first scan in file
        """
        return self._get_wrapped_object_().StartTime

    @start_time.setter
    def start_time(self, value: float):
        """
        Gets or sets the time of first scan in file
        """
        assert type(value) is float
        self._get_wrapped_object_().StartTime = value

    @property
    def end_time(self) -> float:
        """
        Gets or sets the time of last scan in file
        """
        return self._get_wrapped_object_().EndTime

    @end_time.setter
    def end_time(self, value: float):
        """
        Gets or sets the time of last scan in file
        """
        assert type(value) is float
        self._get_wrapped_object_().EndTime = value

    @property
    def low_mass(self) -> float:
        """
        Gets or sets the lowest recorded mass in file
        """
        return self._get_wrapped_object_().LowMass

    @low_mass.setter
    def low_mass(self, value: float):
        """
        Gets or sets the lowest recorded mass in file
        """
        assert type(value) is float
        self._get_wrapped_object_().LowMass = value

    @property
    def high_mass(self) -> float:
        """
        Gets or sets the highest recorded mass in file
        """
        return self._get_wrapped_object_().HighMass

    @high_mass.setter
    def high_mass(self, value: float):
        """
        Gets or sets the highest recorded mass in file
        """
        assert type(value) is float
        self._get_wrapped_object_().HighMass = value

    @property
    def mass_resolution(self) -> float:
        """
        Gets or sets the mass resolution value recorded for the current instrument. The
        value is returned as one half of the mass resolution. For example, a unit resolution
        controller would return a value of 0.5.
        """
        return self._get_wrapped_object_().MassResolution

    @mass_resolution.setter
    def mass_resolution(self, value: float):
        """
        Gets or sets the mass resolution value recorded for the current instrument. The
        value is returned as one half of the mass resolution. For example, a unit resolution
        controller would return a value of 0.5.
        """
        assert type(value) is float
        self._get_wrapped_object_().MassResolution = value

    @property
    def expected_runtime(self) -> float:
        """
        Gets or sets the expected acquisition run time for the current instrument.
        """
        return self._get_wrapped_object_().ExpectedRuntime

    @expected_runtime.setter
    def expected_runtime(self, value: float):
        """
        Gets or sets the expected acquisition run time for the current instrument.
        """
        assert type(value) is float
        self._get_wrapped_object_().ExpectedRuntime = value

    @property
    def max_integrated_intensity(self) -> float:
        """
        Gets or sets the max integrated intensity.
        """
        return self._get_wrapped_object_().MaxIntegratedIntensity

    @max_integrated_intensity.setter
    def max_integrated_intensity(self, value: float):
        """
        Gets or sets the max integrated intensity.
        """
        assert type(value) is float
        self._get_wrapped_object_().MaxIntegratedIntensity = value

    @property
    def max_intensity(self) -> int:
        """
        Gets or sets the max intensity.
        """
        return self._get_wrapped_object_().MaxIntensity

    @max_intensity.setter
    def max_intensity(self, value: int):
        """
        Gets or sets the max intensity.
        """
        assert type(value) is int
        self._get_wrapped_object_().MaxIntensity = value

    @property
    def tolerance_unit(self) -> ToleranceUnits:
        """
        Gets or sets the tolerance unit.
        """
        return ToleranceUnits(self._get_wrapped_object_().ToleranceUnit)

    @tolerance_unit.setter
    def tolerance_unit(self, value: ToleranceUnits):
        """
        Gets or sets the tolerance unit.
        """
        assert type(value) is ToleranceUnits
        self._get_wrapped_object_().ToleranceUnit = value.value



from fisher_py.data.filter_enums.activation_type import ActivationType
from fisher_py.data.filter_enums.compensation_voltage_type import CompensationVoltageType
from fisher_py.data.filter_enums.detector_type import DetectorType
from fisher_py.data.filter_enums.field_free_region_type import FieldFreeRegionType
from fisher_py.data.filter_enums.ionization_mode_type import IonizationModeType
from fisher_py.data.filter_enums.mass_analyzer_type import MassAnalyzerType
from fisher_py.data.filter_enums.ms_order_type import MsOrderType
from fisher_py.data.filter_enums.polarity_type import PolarityType
from fisher_py.data.filter_enums.scan_data_type import ScanDataType
from fisher_py.data.filter_enums.scan_mode_type import ScanModeType
from fisher_py.data.filter_enums.sector_scan_type import SectorScanType
from fisher_py.data.filter_enums.event_accurate_mass import EventAccurateMass
from fisher_py.data.filter_enums.source_fragmentation_value_type import SourceFragmentationValueType
from fisher_py.data.filter_enums.tri_state import TriState
from fisher_py.data.filter_enums.energy_type import EnergyType



from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data import RawFileClassification, ScanDependentDetails
from fisher_py.utils import to_net_list
from typing import List


class ScanDependents(NetWrapperBase):
    """
    The ScanDependents interface. Result of call to "GetScanDependents" Provides
    a set of scan numbers which were created form a particular master scan.
    """

    @property
    def raw_file_instrument_type(self) -> RawFileClassification:
        """
        Gets or sets the type of the raw file instrument.
        
        Value:
        The type of the raw file instrument.
        """
        return RawFileClassification(self._get_wrapped_object_().RawFileInstrumentType)

    @raw_file_instrument_type.setter
    def raw_file_instrument_type(self, value: RawFileClassification):
        """
        Gets or sets the type of the raw file instrument.
        
        Value:
        The type of the raw file instrument.
        """
        assert type(value) is RawFileClassification
        self._get_wrapped_object_().RawFileInstrumentType = value.value

    @property
    def scan_dependent_detail_array(self) -> List[ScanDependentDetails]:
        """
        Gets or sets the scan dependent detail array.
        
        Value:
        The scan dependent detail array.
        """
        return [ScanDependentDetails._get_wrapper_(s) for s in self._get_wrapped_object_().ScanDependentDetailArray]

    @scan_dependent_detail_array.setter
    def scan_dependent_detail_array(self, value: List[ScanDependentDetails]):
        """
        Gets or sets the scan dependent detail array.
        
        Value:
        The scan dependent detail array.
        """
        assert type(value) is list
        value = to_net_list(value, type(value[0]._get_wrapped_object_()) if len(value) > 0 else object)
        self._get_wrapped_object_().ScanDependentDetailArray = [s._get_wrapped_object_() for s in value]




import enum


class TraceType(enum.Enum):
    """
     Enumeration of trace types, for chromatograms. Note: legacy C++ file reader does
    not support analog trace numbers above "4" or UV above "channel D". Traces are
    organized in blocks of 10 For example: StartAnalogChromatogramTraces=10 (not
    a valid trace type, just a limit) Analog1 to Analog8 = 11 to 18 EndAnalogChromatogramTraces
    =19 (not a valid trace type, just a limit) Next block: StartPDAChromatogramTraces
    = 20 Etc.
    """
    StartMSChromatogramTraces = -1
    MassRange = 0
    TIC = 1
    BasePeak = 2
    Fragment = 3
    EndMSChromatogramTraces = 4
    StartAnalogChromatogramTraces = 10
    Analog1 = 11
    Analog2 = 12
    Analog3 = 13
    Analog4 = 14
    Analog5 = 15
    Analog6 = 16
    Analog7 = 17
    Analog8 = 18
    EndAnalogChromatogramTraces = 19
    StartPDAChromatogramTraces = 20
    WavelengthRange = 21
    TotalAbsorbance = 22
    SpectrumMax = 23
    EndPDAChromatogramTraces = 24
    StartUVChromatogramTraces = 30
    ChannelA = 31
    ChannelB = 32
    ChannelC = 33
    ChannelD = 34
    ChannelE = 35
    ChannelF = 36
    ChannelG = 37
    ChannelH = 38
    EndUVChromatogramTraces = 39
    StartPCA2DChromatogramTraces = 40
    A2DChannel1 = 41
    A2DChannel2 = 42
    A2DChannel3 = 43
    ChromatogramA2DChannel3 = 43
    A2DChannel4 = 44
    ChromatogramA2DChannel4 = 44
    A2DChannel5 = 45
    A2DChannel6 = 46
    A2DChannel7 = 47
    A2DChannel8 = 48
    EndPCA2DChromatogramTraces = 49
    EndAllChromatogramTraces = 50



import enum


class SourceFragmentationValueType(enum.Enum):
    """
    Specifies how source fragmentation values are interpreted.
    """
    NoValue = 0
    SingleValue = 1
    Ramp = 2
    SIM = 3
    Any = 4



import enum


class ActivationType(enum.Enum):
    """
    The activation types are used to link a specific precursor mass with an activation
    type. There are 26 possible mode values, including some reserved values.
    """
    CollisionInducedDissociation = 0
    MultiPhotonDissociation = 1
    ElectronCaptureDissociation = 2
    PQD = 3
    ElectronTransferDissociation = 4
    HigherEnergyCollisionalDissociation = 5
    Any = 6
    SAactivation = 7
    ProtonTransferReaction = 8
    NegativeElectronTransferDissociation = 9
    NegativeProtonTransferReaction = 10
    UltraVioletPhotoDissociation = 11
    ModeA = 12
    ModeB = 13
    ModeC = 14
    ModeD = 15
    ModeE = 16
    ModeF = 17
    ModeG = 18
    ModeH = 19
    ModeI = 20
    ModeJ = 21
    ModeK = 22
    ModeL = 23
    ModeM = 24
    ModeN = 25
    ModeO = 26
    ModeP = 27
    ModeQ = 28
    ModeR = 29
    ModeS = 30
    ModeT = 31
    ModeU = 32
    ModeV = 33
    ModeW = 34
    ModeX = 35
    ModeY = 36
    ModeZ = 37
    LastActivation = 38


from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.utils import is_number


class EstimatorResults(NetWrapperBase):
    """
    Class to hold mass precision estimator results for individual mass/intensity
    points in a scan.
    """

    _wrapped_type = ThermoFisher.CommonCore.MassPrecisionEstimator.EstimatorResults

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_object()

    @property
    def intensity(self) -> float:
        """
        Gets or sets the intensity value
        """
        return self._get_wrapped_object_().Intensity

    @intensity.setter
    def intensity(self, value: float):
        """
        Gets or sets the intensity value
        """
        assert type(value) is float
        self._get_wrapped_object_().Intensity = value

    @property
    def mass(self) -> float:
        """
        Gets or sets the mass value
        """
        return self._get_wrapped_object_().Mass

    @mass.setter
    def mass(self, value: float):
        """
        Gets or sets the mass value
        """
        assert is_number(value)
        self._get_wrapped_object_().Mass = float(value)

    @property
    def mass_accuracy_in_mmu(self) -> float:
        """
        Gets or sets the mass accuracy in MMU value
        """
        return self._get_wrapped_object_().MassAccuracyInMmu

    @mass_accuracy_in_mmu.setter
    def mass_accuracy_in_mmu(self, value: float):
        """
        Gets or sets the mass accuracy in MMU value
        """
        assert is_number(value)
        self._get_wrapped_object_().MassAccuracyInMmu = float(value)

    @property
    def mass_accuracy_in_ppm(self) -> float:
        """
        Gets or sets the mass accuracy in PPM value
        """
        return self._get_wrapped_object_().MassAccuracyInPpm

    @mass_accuracy_in_ppm.setter
    def mass_accuracy_in_ppm(self, value: float):
        """
        Gets or sets the mass accuracy in PPM value
        """
        assert is_number(value)
        self._get_wrapped_object_().MassAccuracyInPpm = float(value)

    @property
    def resolution(self) -> float:
        """
        Gets or sets the resolution value
        """
        return self._get_wrapped_object_().Resolution

    @resolution.setter
    def resolution(self, value: float):
        """
        Gets or sets the resolution value
        """
        assert is_number(value)
        self._get_wrapped_object_().Resolution = float(value)



import enum


class IonizationModeType(enum.Enum):
    """
    Specifies ionization mode in scans.
    """
    ElectronImpact = 0
    ChemicalIonization = 1
    FastAtomBombardment = 2
    ElectroSpray = 3
    AtmosphericPressureChemicalIonization = 4
    NanoSpray = 5
    ThermoSpray = 6
    FieldDesorption = 7
    MatrixAssistedLaserDesorptionIonization = 8
    GlowDischarge = 9
    Any = 10
    PaperSprayIonization = 11
    CardNanoSprayIonization = 12
    IonizationMode1 = 13
    IonizationMode2 = 14
    IonizationMode3 = 15
    IonizationMode4 = 16
    IonizationMode5 = 17
    IonizationMode6 = 18
    IonizationMode7 = 19
    IonizationMode8 = 20
    IonizationMode9 = 21
    IonModeBeyondKnown = 22



from typing import List
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.utils import is_number, to_net_list
from fisher_py.data.filter_enums import (
    SectorScanType, FieldFreeRegionType, TriState, CompensationVoltageType, ScanDataType, 
    PolarityType, SourceFragmentationValueType, ScanModeType, MassAnalyzerType, DetectorType,
    IonizationModeType, MsOrderType
)
from fisher_py.data import FilterAccurateMass, SourceFragmentationInfoValidType


class ScanFilter(NetWrapperBase):
    """
    The ScanFilter interface defines a set of rules for selecting scans. For example:
    Testing if a scan should be included in a chromatogram. Many if the rules include
    an "Any" choice, which implies that this rule will not be tested. Testing logic
    is included in the class ThermoFisher.CommonCore.Data.Business.ScanFilterHelper
    The class ThermoFisher.CommonCore.Data.Extensions contains methods related to
    filters, the filter helper and raw data.
    """

    def __init__(self, scan_filter_net):
        super().__init__()
        self._wrapped_object = scan_filter_net

    @property
    def sector_scan(self) -> SectorScanType:
        """
        Gets or sets the sector scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SectorScanType for possible values
        """
        return SectorScanType(self._get_wrapped_object_().SectorScan)

    @sector_scan.setter
    def sector_scan(self, value: SectorScanType):
        """
        Gets or sets the sector scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SectorScanType for possible values
        """
        assert type(value) is SectorScanType
        self._get_wrapped_object_().SectorScan = value.value

    @property
    def field_free_region(self) -> FieldFreeRegionType:
        """
        Gets or sets the field free region filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.FieldFreeRegionType for possible
        values
        """
        return FieldFreeRegionType(self._get_wrapped_object_().FieldFreeRegion)

    @field_free_region.setter
    def field_free_region(self, value: FieldFreeRegionType):
        """
        Gets or sets the field free region filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.FieldFreeRegionType for possible
        values
        """
        assert type(value) is FieldFreeRegionType
        self._get_wrapped_object_().FieldFreeRegion = value.value

    @property
    def ultra(self) -> TriState:
        """
        Gets or sets the ultra scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().Ultra)

    @ultra.setter
    def ultra(self, value: TriState):
        """
        Gets or sets the ultra scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Ultra = value.value

    @property
    def enhanced(self) -> TriState:
        """
        Gets or sets the enhanced scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().Enhanced)

    @enhanced.setter
    def enhanced(self, value: TriState):
        """
        Gets or sets the enhanced scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Enhanced = value.value

    @property
    def multiple_photon_dissociation(self) -> TriState:
        """
        Gets or sets the multi-photon dissociation filtering rule.
        """
        return TriState(self._get_wrapped_object_().MultiplePhotonDissociation)

    @multiple_photon_dissociation.setter
    def multiple_photon_dissociation(self, value: TriState):
        """
        Gets or sets the multi-photon dissociation filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().MultiplePhotonDissociation = value.value

    @property
    def multiple_photon_dissociation_value(self) -> float:
        """
        Gets or sets the multi-photon dissociation value.
        
        Value:
        Floating point multi-photon dissociation value
        """
        return self._get_wrapped_object_().MultiplePhotonDissociationValue

    @multiple_photon_dissociation_value.setter
    def multiple_photon_dissociation_value(self, value: float):
        """
        Gets or sets the multi-photon dissociation value.
        
        Value:
        Floating point multi-photon dissociation value
        """
        assert is_number(value)
        self._get_wrapped_object_().MultiplePhotonDissociationValue = float(value)

    @property
    def electron_capture_dissociation(self) -> TriState:
        """
        Gets or sets the electron capture dissociation filtering rule.
        """
        return TriState(self._get_wrapped_object_().ElectronCaptureDissociation)

    @electron_capture_dissociation.setter
    def electron_capture_dissociation(self, value: TriState):
        """
        Gets or sets the electron capture dissociation filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ElectronCaptureDissociation = value.value

    @property
    def electron_capture_dissociation_value(self) -> float:
        """
        Gets or sets the electron capture dissociation value.
        
        Value:
        Floating point electron capture dissociation value
        """
        return self._get_wrapped_object_().ElectronCaptureDissociationValue

    @electron_capture_dissociation_value.setter
    def electron_capture_dissociation_value(self, value: float):
        """
        Gets or sets the electron capture dissociation value.
        
        Value:
        Floating point electron capture dissociation value
        """
        assert is_number(value)
        self._get_wrapped_object_().ElectronCaptureDissociationValue = float(value)

    @property
    def photo_ionization(self) -> TriState:
        """
        Gets or sets the photo ionization filtering rule.
        """
        return TriState(self._get_wrapped_object_().PhotoIonization)

    @photo_ionization.setter
    def photo_ionization(self, value: TriState):
        """
        Gets or sets the photo ionization filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().PhotoIonization = value.value

    @property
    def pulsed_q_dissociation(self) -> TriState:
        """
        Gets or sets pulsed dissociation filtering rule.
        """
        return TriState(self._get_wrapped_object_().PulsedQDissociation)

    @pulsed_q_dissociation.setter
    def pulsed_q_dissociation(self, value: TriState):
        """
        Gets or sets pulsed dissociation filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().PulsedQDissociation = value.value

    @property
    def pulsed_q_dissociation_value(self) -> float:
        """
        Gets or sets the pulsed dissociation value. Only applies when the PulsedQDissociation
        rule is used.
        
        Value:
        Floating point pulsed dissociation value
        """
        return self._get_wrapped_object_().PulsedQDissociationValue

    @pulsed_q_dissociation_value.setter
    def pulsed_q_dissociation_value(self, value: float):
        """
        Gets or sets the pulsed dissociation value. Only applies when the PulsedQDissociation
        rule is used.
        
        Value:
        Floating point pulsed dissociation value
        """
        assert is_number(value)
        self._get_wrapped_object_().PulsedQDissociationValue = float(value)

    @property
    def electron_transfer_dissociation(self) -> TriState:
        """
        Gets or sets the electron transfer dissociation filtering rule.
        """
        return TriState(self._get_wrapped_object_().ElectronTransferDissociation)

    @electron_transfer_dissociation.setter
    def electron_transfer_dissociation(self, value: TriState):
        """
        Gets or sets the electron transfer dissociation filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ElectronTransferDissociation = value.value

    @property
    def lock(self) -> TriState:
        """
        Gets or sets the lock scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().Lock)

    @lock.setter
    def lock(self, value: TriState):
        """
        Gets or sets the lock scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Lock = value.value

    @property
    def electron_transfer_dissociation_value(self) -> float:
        """
        Gets or sets the electron transfer dissociation value. Only used when the "ElectronTransferDissociation"
        rule is used.
        
        Value:
        Floating point electron transfer dissociation value
        """
        return self._get_wrapped_object_().ElectronTransferDissociationValue

    @electron_transfer_dissociation_value.setter
    def electron_transfer_dissociation_value(self, value: float):
        """
        Gets or sets the electron transfer dissociation value. Only used when the "ElectronTransferDissociation"
        rule is used.
        
        Value:
        Floating point electron transfer dissociation value
        """
        assert is_number(value)
        self._get_wrapped_object_().ElectronTransferDissociationValue = float(value)

    @property
    def higher_energy_ci_d_value(self) -> float:
        """
        Gets or sets the higher energy CID value. Only applies when the "HigherEnergyCiD"
        rule is used.
        """
        return self._get_wrapped_object_().HigherEnergyCiDValue

    @higher_energy_ci_d_value.setter
    def higher_energy_ci_d_value(self, value: float):
        """
        Gets or sets the higher energy CID value. Only applies when the "HigherEnergyCiD"
        rule is used.
        """
        assert is_number(value)
        self._get_wrapped_object_().HigherEnergyCiDValue = float(value)

    @property
    def multiplex(self) -> TriState:
        """
        Gets or sets the Multiplex type filtering rule.
        """
        return TriState(self._get_wrapped_object_().Multiplex)

    @multiplex.setter
    def multiplex(self, value: TriState):
        """
        Gets or sets the Multiplex type filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Multiplex = value.value

    @property
    def param_a(self) -> TriState:
        """
        Gets or sets the parameter a filtering rule..
        """
        return TriState(self._get_wrapped_object_().ParamA)

    @param_a.setter
    def param_a(self, value: TriState):
        """
        Gets or sets the parameter a filtering rule..
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ParamA = value.value

    @property
    def param_b(self) -> TriState:
        """
        Gets or sets the parameter b filtering rule..
        """
        return TriState(self._get_wrapped_object_().ParamB)

    @param_b.setter
    def param_b(self, value: TriState):
        """
        Gets or sets the parameter b filtering rule..
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ParamB = value.value

    @property
    def param_f(self) -> TriState:
        """
        Gets or sets the parameter f filtering rule..
        """
        return TriState(self._get_wrapped_object_().ParamF)

    @param_f.setter
    def param_f(self, value: TriState):
        """
        Gets or sets the parameter f filtering rule..
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ParamF = value.value

    @property
    def multi_notch(self) -> TriState:
        """
        Gets or sets the SPS (Synchronous Precursor Selection) Multi notch filtering
        rule.
        """
        return TriState(self._get_wrapped_object_().MultiNotch)

    @multi_notch.setter
    def multi_notch(self, value: TriState):
        """
        Gets or sets the SPS (Synchronous Precursor Selection) Multi notch filtering
        rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().MultiNotch = value.value

    @property
    def param_r(self) -> TriState:
        """
        Gets or sets the parameter r filtering rule..
        """
        return TriState(self._get_wrapped_object_().ParamR)

    @param_r.setter
    def param_r(self, value: TriState):
        """
        Gets or sets the parameter r filtering rule..
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ParamR = value.value

    @property
    def param_v(self) -> TriState:
        """
        Gets or sets the parameter v filtering rule..
        """
        return TriState(self._get_wrapped_object_().ParamV)

    @param_v.setter
    def param_v(self, value: TriState):
        """
        Gets or sets the parameter v filtering rule..
        """
        assert type(value) is TriState
        self._get_wrapped_object_().ParamV = value.value

    @property
    def name(self) -> str:
        """
        Gets or sets the event Name. Used for "compound name" filtering.
        """
        return self._get_wrapped_object_().Name

    @name.setter
    def name(self, value: str):
        """
        Gets or sets the event Name. Used for "compound name" filtering.
        """
        assert type(value) is str
        self._get_wrapped_object_().Name = value

    @property
    def supplemental_activation(self) -> TriState:
        """
        Gets or sets supplemental activation type filter rule.
        """
        return TriState(self._get_wrapped_object_().SupplementalActivation)

    @supplemental_activation.setter
    def supplemental_activation(self, value: TriState):
        """
        Gets or sets supplemental activation type filter rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().SupplementalActivation = value.value

    @property
    def multi_state_activation(self) -> TriState:
        """
        Gets or sets MultiStateActivation type filtering rule.
        """
        return TriState(self._get_wrapped_object_().MultiStateActivation)

    @multi_state_activation.setter
    def multi_state_activation(self, value: TriState):
        """
        Gets or sets MultiStateActivation type filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().MultiStateActivation = value.value

    @property
    def higher_energy_ci_d(self) -> TriState:
        """
        Gets or sets the higher energy CID filtering rule.
        """
        return TriState(self._get_wrapped_object_().HigherEnergyCiD)

    @higher_energy_ci_d.setter
    def higher_energy_ci_d(self, value: TriState):
        """
        Gets or sets the higher energy CID filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().HigherEnergyCiD = value.value

    @property
    def compensation_voltage(self) -> TriState:
        """
        Gets or sets Compensation Voltage filtering rule.
        """
        return TriState(self._get_wrapped_object_().CompensationVoltage)

    @compensation_voltage.setter
    def compensation_voltage(self, value: TriState):
        """
        Gets or sets Compensation Voltage filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().CompensationVoltage = value.value

    @property
    def compensation_volt_type(self) -> CompensationVoltageType:
        """
        Gets or sets Compensation Voltage type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.CompensationVoltageType for possible
        values
        """
        return CompensationVoltageType(self._get_wrapped_object_().CompensationVoltType)

    @compensation_volt_type.setter
    def compensation_volt_type(self, value: CompensationVoltageType):
        """
        Gets or sets Compensation Voltage type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.CompensationVoltageType for possible
        values
        """
        assert type(value) is CompensationVoltageType
        self._get_wrapped_object_().CompensationVoltType = value.value

    @property
    def detector_value(self) -> float:
        """
        Gets or sets the detector value. This is used for filtering when the Detector
        filter is enabled.
        
        Value:
        Floating point detector value
        """
        return self._get_wrapped_object_().DetectorValue

    @detector_value.setter
    def detector_value(self, value: float):
        """
        Gets or sets the detector value. This is used for filtering when the Detector
        filter is enabled.
        
        Value:
        Floating point detector value
        """
        assert is_number(value)
        self._get_wrapped_object_().DetectorValue = float(value)

    @property
    def accurate_mass(self) -> FilterAccurateMass:
        """
        Gets the accurate mass filter rule.
        """
        return FilterAccurateMass(self._get_wrapped_object_().AccurateMass)

    @property
    def mass_precision(self) -> int:
        """
        Gets or sets the mass precision, which is used to format the filter (in ToString).
        """
        return self._get_wrapped_object_().MassPrecision

    @mass_precision.setter
    def mass_precision(self, value: int):
        """
        Gets or sets the mass precision, which is used to format the filter (in ToString).
        """
        assert type(value) is int
        self._get_wrapped_object_().MassPrecision = value

    @property
    def meta_filters(self) -> int:
        """
        Gets or sets additional instrument defined filters (these are bit flags). See
        enum MetaFilterType.
        """
        return self._get_wrapped_object_().MetaFilters

    @meta_filters.setter
    def meta_filters(self, value: int):
        """
        Gets or sets additional instrument defined filters (these are bit flags). See
        enum MetaFilterType.
        """
        assert type(value) is int
        self._get_wrapped_object_().MetaFilters = value

    @property
    def unique_mass_count(self) -> int:
        """
        Gets the number of unique masses, taking into account multiple activations. For
        example: If this is MS3 data, there are two "parent masses", but they may have
        multiple reactions applied. If the first stage has two reactions, then there
        are a total of 3 reactions, for the 2 "unique masses"
        """
        return self._get_wrapped_object_().UniqueMassCount

    @property
    def source_fragmentation_info_valid(self) -> List[SourceFragmentationInfoValidType]:
        """
        Gets or sets an array of values which determines if the source fragmentation
        values are valid.
        """
        return [SourceFragmentationInfoValidType(i) for i in self._get_wrapped_object_().SourceFragmentationInfoValid]

    @source_fragmentation_info_valid.setter
    def source_fragmentation_info_valid(self, value: List[SourceFragmentationInfoValidType]):
        """
        Gets or sets an array of values which determines if the source fragmentation
        values are valid.
        """
        assert type(value) is list
        value = to_net_list([i._get_wrapped_object_() for i in value], type(value[0]._get_wrapped_object_()))
        self._get_wrapped_object_().SourceFragmentationInfoValid = value

    @property
    def locale_name(self) -> str:
        """
        Gets or sets the locale name. This can be used to affect string conversion.
        """
        return self._get_wrapped_object_().LocaleName

    @locale_name.setter
    def locale_name(self, value: str):
        """
        Gets or sets the locale name. This can be used to affect string conversion.
        """
        assert type(value) is str
        self._get_wrapped_object_().LocaleName = value

    @property
    def compensation_voltage_count(self) -> int:
        """
        Gets the number of compensation voltage values. This is the number of values
        related to cv mode. 1 for "single value", 2 for "ramp" and 1 per mass range for
        "SIM".
        """
        return self._get_wrapped_object_().CompensationVoltageCount

    @property
    def wideband(self) -> TriState:
        """
        Gets or sets the wideband filtering rule.
        """
        return TriState(self._get_wrapped_object_().Wideband)

    @wideband.setter
    def wideband(self, value: TriState):
        """
        Gets or sets the wideband filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Wideband = value.value

    @property
    def scan_data(self) -> ScanDataType:
        """
        Gets or sets the scan data type (centroid or profile) filtering rule.
        """
        return ScanDataType(self._get_wrapped_object_().ScanData)

    @scan_data.setter
    def scan_data(self, value: ScanDataType):
        """
        Gets or sets the scan data type (centroid or profile) filtering rule.
        """
        assert type(value) is ScanDataType
        self._get_wrapped_object_().ScanData = value.value

    @property
    def polarity(self) -> PolarityType:
        """
        Gets or sets the polarity (+/-) filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.PolarityType for possible values
        """
        return PolarityType(self._get_wrapped_object_().Polarity)

    @polarity.setter
    def polarity(self, value: PolarityType):
        """
        Gets or sets the polarity (+/-) filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.PolarityType for possible values
        """
        assert type(value) is PolarityType
        self._get_wrapped_object_().Polarity = value.value

    @property
    def souce_fragmentaion_value_count(self) -> int:
        """
        Gets the number of Source Fragmentation values. This is the number of values
        related to sid mode. 1 for "single value", 2 for "ramp" and 1 per mass range
        for "SIM".
        """
        return self._get_wrapped_object_().SouceFragmentaionValueCount

    @property
    def dependent(self) -> TriState:
        """
        Gets or sets the dependent scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().Dependent)

    @dependent.setter
    def dependent(self, value: TriState):
        """
        Gets or sets the dependent scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Dependent = value.value

    @property
    def source_fragmentation(self) -> TriState:
        """
        Gets or sets source fragmentation scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().SourceFragmentation)

    @source_fragmentation.setter
    def source_fragmentation(self, value: TriState):
        """
        Gets or sets source fragmentation scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().SourceFragmentation = value.value

    @property
    def source_fragmentation_type(self) -> SourceFragmentationValueType:
        """
        Gets or sets the source fragmentation type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SourceFragmentationValueType for
        possible values
        """
        return SourceFragmentationInfoValidType(self._get_wrapped_object_().SourceFragmentationType)

    @source_fragmentation_type.setter
    def source_fragmentation_type(self, value: SourceFragmentationValueType):
        """
        Gets or sets the source fragmentation type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SourceFragmentationValueType for
        possible values
        """
        assert type(value) is SourceFragmentationValueType
        self._get_wrapped_object_().SourceFragmentationType = value.value

    @property
    def scan_mode(self) -> ScanModeType:
        """
        Gets or sets the scan type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.ScanModeType for possible values
        """
        return ScanModeType(self._get_wrapped_object_().ScanMode)

    @scan_mode.setter
    def scan_mode(self, value: ScanModeType):
        """
        Gets or sets the scan type filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.ScanModeType for possible values
        """
        assert type(value) is ScanModeType
        self._get_wrapped_object_().ScanMode = value.value

    @property
    def mass_analyzer(self) -> MassAnalyzerType:
        """
        Gets or sets the mass analyzer scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MassAnalyzerType for possible values
        """
        return MassAnalyzerType(self._get_wrapped_object_().MassAnalyzer)

    @mass_analyzer.setter
    def mass_analyzer(self, value: MassAnalyzerType):
        """
        Gets or sets the mass analyzer scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MassAnalyzerType for possible values
        """
        assert type(value) is MassAnalyzerType
        self._get_wrapped_object_().MassAnalyzer = value.value

    @property
    def detector(self) -> DetectorType:
        """
        Gets or sets the detector scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.DetectorType for possible values
        """
        return DetectorType(self._get_wrapped_object_().Detector)

    @detector.setter
    def detector(self, value: DetectorType):
        """
        Gets or sets the detector scan filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.DetectorType for possible values
        """
        assert type(value) is DetectorType
        self._get_wrapped_object_().Detector = value.value

    @property
    def turbo_scan(self) -> TriState:
        """
        Gets or sets the turbo scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().TurboScan)

    @turbo_scan.setter
    def turbo_scan(self, value: TriState):
        """
        Gets or sets the turbo scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().TurboScan = value.value

    @property
    def ionization_mode(self) -> IonizationModeType:
        """
        Gets or sets the ionization mode filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.IonizationModeType for possible
        values
        """
        return IonizationModeType(self._get_wrapped_object_().IonizationMode)

    @ionization_mode.setter
    def ionization_mode(self, value: IonizationModeType):
        """
        Gets or sets the ionization mode filtering rule.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.IonizationModeType for possible
        values
        """
        assert type(value) is IonizationModeType
        self._get_wrapped_object_().IonizationMode = value.value

    @property
    def corona(self) -> TriState:
        """
        Gets or sets the corona scan filtering rule.
        """
        return TriState(self._get_wrapped_object_().Corona)

    @corona.setter
    def corona(self, value: TriState):
        """
        Gets or sets the corona scan filtering rule.
        """
        assert type(value) is TriState
        self._get_wrapped_object_().Corona = value.value

    @property
    def ms_order(self) -> MsOrderType:
        """
        Gets or sets the scan power or MS/MS mode filtering rule, such as MS3 or Parent
        scan.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MSOrderType for possible values
        """
        return MsOrderType(self._get_wrapped_object_().MSOrder)

    @ms_order.setter
    def ms_order(self, value: MsOrderType):
        """
        Gets or sets the scan power or MS/MS mode filtering rule, such as MS3 or Parent
        scan.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MSOrderType for possible values
        """
        assert type(value) is MsOrderType
        self._get_wrapped_object_().MSOrder = value.value

    def compensation_voltage_value(self, index: int) -> float:
        """
        Retrieves a compensation voltage (cv) value at 0-based index.
        
        Parameters:
        index:
        Index of compensation voltage to be retrieved
        
        Returns:
        Compensation voltage value (cv) at 0-based index
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanFilter.CompensationVoltageCount
        to get the count of compensation voltage values.
        """
        return self._get_wrapped_object_().CompensationVoltageValue(index)

    def get_source_fragmentation_info_valid(self, index: int) -> SourceFragmentationInfoValidType:
        """
        Get source fragmentation info valid, at zero based index.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The ThermoFisher.CommonCore.Data.Interfaces.SourceFragmentationInfoValidType.
        """
        return self._get_wrapped_object_().GetSourceFragmentationInfoValid(index)

    def index_to_multiple_activation_index(self, index: int) -> int:
        """
        Convert an index to multiple activation index. Converts a simple mass index to
        the index to the unique mass, taking into account multiple activations.
        
        Parameters:
        index:
        The index to convert.
        
        Returns:
        The index of the unique mass.
        """
        return self._get_wrapped_object_().IndexToMultipleActivationIndex(index)

    def source_fragmentation_value(self, index: int) -> float:
        """
        Retrieves a source fragmentation value (sid) at 0-based index.
        
        Parameters:
        index:
        Index of source fragmentation value to be retrieved
        
        Returns:
        Source Fragmentation Value (sid) at 0-based index
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanFilter.SouceFragmentaionValueCount
        to get the count of source fragmentation values.
        """
        return self._get_wrapped_object_().SourceFragmentationValue(index)

    def __str__(self) -> str:
        """
        Convert to string. Mass values are converted as per the precision.
        
        Returns:
        The System.String.
        """
        return self._get_wrapped_object_().ToString()



from __future__ import annotations
from fisher_py.data.business.range import Range
from typing import List, Any, TYPE_CHECKING
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data import PeakOptions
from fisher_py.data.business import (
    MassToFrequencyConverter, NoiseAndBaseline, ScanStatistics, SegmentedScan, ToleranceMode, 
    CentroidStream, CachedScanProvider
)
from fisher_py.utils import to_net_list

if TYPE_CHECKING:
    from fisher_py.raw_file_reader import RawFileAccess


class Scan(NetWrapperBase):
    """
    Class to represent a scan
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.Scan

    def __init__(self, *args):
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        else:
            if len(args) == 1:
                converter = args[0]
                raw_file_noise = None
            elif len(args) == 2:
                converter, raw_file_noise = args
                assert type(raw_file_noise) is list
                if len(raw_file_noise) > 0:
                    assert type(raw_file_noise[0]) is NoiseAndBaseline
                    raw_file_noise = to_net_list(raw_file_noise, NoiseAndBaseline._wrapped_type)
            else:
                raise ValueError('Unable to create Scan.')

            assert type(converter) is MassToFrequencyConverter
            self._wrapped_object(converter._get_wrapped_object_(), [o._get_wrapped_object_() for o in raw_file_noise])

    @property
    def preferred_base_peak_mass(self) -> float:
        """
        Gets Mass of base peak default data stream (usually centroid stream, if present).
        Falls back to ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan data if
        centroid stream is not preferred or not present
        """
        return self._get_wrapped_object_().PreferredBasePeakMass

    @property
    def preferred_base_peak_noise(self) -> float:
        """
        Gets Noise of base peak for default data stream (usually centroid stream, if
        present). Falls back to zero if centroid stream is not preferred or not present
        """
        return self._get_wrapped_object_().PreferredBasePeakNoise

    @property
    def preferred_base_peak_resolution(self) -> float:
        """
        Gets Resolution of base peak for default data stream (usually centroid stream,
        if present). Falls back to zero if centroid stream is not preferred or not present
        """
        return self._get_wrapped_object_().PreferredBasePeakResolution

    @property
    def preferred_flags(self) -> List[PeakOptions]:
        """
        Gets peak flags (such as saturated) for default data stream (usually centroid
        stream, if present). Falls back to ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan
        data if centroid stream is not preferred or not present
        """
        return list(self._get_wrapped_object_().PreferredFlags)

    @property
    def preferred_intensities(self) -> List[float]:
        """
        Gets Intensity for default data stream (usually centroid stream, if present).
        Falls back to ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan data if
        centroid stream is not preferred or not present
        """
        return list(self._get_wrapped_object_().PreferredIntensities)

    @property
    def preferred_masses(self) -> List[float]:
        """
        Gets the Mass for default data stream (usually centroid stream, if present).
        Falls back to ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan data if
        centroid stream is not preferred or not present
        """
        return list(self._get_wrapped_object_().PreferredMasses)

    @property
    def preferred_noises(self) -> List[float]:
        """
        Gets Noises for default data stream (usually centroid stream, if present). Returns
        an empty array if centroid stream is not preferred or not present
        """
        return list(self._get_wrapped_object_().PreferredNoises)

    @property
    def preferred_baselines(self) -> List[float]:
        """
        Gets Baselines for default data stream (usually centroid stream, if present).
        Returns an empty array if centroid stream is not preferred or not present
        """
        return list(self._get_wrapped_object_().PreferredBaselines)

    @property
    def scans_combined(self) -> int:
        """
        Gets or sets the number of scans which were combined to create this scan. For
        example: By the scan averager. This can be zero if this is a "scan read from
        a file"
        """
        return self._get_wrapped_object_().ScansCombined

    @scans_combined.setter
    def scans_combined(self, value: int):
        """
        Gets or sets the number of scans which were combined to create this scan. For
        example: By the scan averager. This can be zero if this is a "scan read from
        a file"
        """
        assert type(value) is int
        self._get_wrapped_object_().ScansCombined = value

    @property
    def preferred_base_peak_intensity(self) -> float:
        """
        Gets peak flags (such as saturated) for default data stream (usually centroid
        stream, if present). Falls back to ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan
        data if centroid stream is not preferred or not present
        """
        return self._get_wrapped_object_().PreferredBasePeakIntensity

    @property
    def scan_statistics(self) -> ScanStatistics:
        """
        Gets or sets Header information for the scan
        """
        return ScanStatistics._get_wrapper_(self._get_wrapped_object_().ScanStatistics)

    @scan_statistics.setter
    def scan_statistics(self, value: ScanStatistics):
        """
        Gets or sets Header information for the scan
        """
        assert type(value) is ScanStatistics
        self._get_wrapped_object_().ScanStatistics = value._get_wrapped_object_()

    @property
    def scan_statistics_access(self) -> ScanStatistics:
        """
        Gets Header information for the scan
        """
        return ScanStatistics._get_wrapper_(self._get_wrapped_object_().ScanStatisticsAccess)

    @property
    def scan_type(self) -> str:
        """
        Gets or sets Type of scan (for filtering)
        """
        return self._get_wrapped_object_().ScanType

    @scan_type.setter
    def scan_type(self, value: str):
        """
        Gets or sets Type of scan (for filtering)
        """
        assert type(value) is str
        self._get_wrapped_object_().ScanType = value

    @property
    def segmented_scan(self) -> SegmentedScan:
        """
        Gets or sets The data for the scan
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object_().SegmentedScan)

    @segmented_scan.setter
    def segmented_scan(self, value: SegmentedScan):
        """
        Gets or sets The data for the scan
        """
        assert type(value) is SegmentedScan
        self._get_wrapped_object_().SegmentedScan = value._get_wrapped_object_()

    @property
    def segmented_scan_access(self) -> SegmentedScan:
        """
        Gets The data for the scan
        """
        return SegmentedScan._get_wrapper_(self._get_wrapped_object_().SegmentedScanAccess)

    @property
    def subtraction_pointer(self) -> Any:
        """
        Gets or sets IScanSubtract interface pointer.
        
        Returns:
        Interface to perform subtraction
        """
        return self._get_wrapped_object_().SubtractionPointer

    @subtraction_pointer.setter
    def subtraction_pointer(self, value: Any):
        """
        Gets or sets IScanSubtract interface pointer.
        
        Returns:
        Interface to perform subtraction
        """
        self._get_wrapped_object_().SubtractionPointer = value

    @property
    def scan_adder(self) -> Any:
        """
        Gets or sets IScanAdd interface. This delegates addition of FT profile scans.
        
        Returns:
        Interface to perform addition
        """
        return self._get_wrapped_object_().ScanAdder

    @scan_adder.setter
    def scan_adder(self, value: Any):
        """
        Gets or sets IScanAdd interface. This delegates addition of FT profile scans.
        
        Returns:
        Interface to perform addition
        """
        self._get_wrapped_object_().ScanAdder = value

    @property
    def preferred_resolutions(self) -> List[float]:
        """
        Gets Resolutions for default data stream (usually centroid stream, if present).
        Returns an empty array if centroid stream is not preferred or not present
        """
        return self._get_wrapped_object_().PreferredResolutions

    @property
    def prefer_centroids(self) -> bool:
        """
        Gets or sets a value indicating whether, when requesting "Preferred data", the
        centroid stream will be returned. For example "ThermoFisher.CommonCore.Data.Business.Scan.PreferredMasses",
        "ThermoFisher.CommonCore.Data.Business.Scan.PreferredIntensities". If this property
        is false, or there is no centroid stream, then these methods will return the
        data from ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan. For greater
        efficiency, callers should cache the return of "ThermoFisher.CommonCore.Data.Business.Scan.PreferredMasses".
        Typically data processing, such as elemental compositions, should use these methods.
        """
        return self._get_wrapped_object_().PreferCentroids

    @prefer_centroids.setter
    def prefer_centroids(self, value: bool):
        """
        Gets or sets a value indicating whether, when requesting "Preferred data", the
        centroid stream will be returned. For example "ThermoFisher.CommonCore.Data.Business.Scan.PreferredMasses",
        "ThermoFisher.CommonCore.Data.Business.Scan.PreferredIntensities". If this property
        is false, or there is no centroid stream, then these methods will return the
        data from ThermoFisher.CommonCore.Data.Business.Scan.SegmentedScan. For greater
        efficiency, callers should cache the return of "ThermoFisher.CommonCore.Data.Business.Scan.PreferredMasses".
        Typically data processing, such as elemental compositions, should use these methods.
        """
        assert type(value) is bool
        self._get_wrapped_object_().PreferCentroids = value

    @property
    def is_user_tolerance(self) -> bool:
        """
        Gets or sets a value indicating whether the User Tolerance value is being used.
        """
        return self._get_wrapped_object_().IsUserTolerance

    @is_user_tolerance.setter
    def is_user_tolerance(self, value: bool):
        """
        Gets or sets a value indicating whether the User Tolerance value is being used.
        """
        assert type(value) is bool
        self._get_wrapped_object_().IsUserTolerance = value

    @property
    def tolerance_unit(self) -> ToleranceMode:
        """
        Gets or sets the Tolerance value.
        """
        return ToleranceMode(self._get_wrapped_object_().ToleranceUnit)

    @tolerance_unit.setter
    def tolerance_unit(self, value: ToleranceMode):
        """
        Gets or sets the Tolerance value.
        """
        assert type(value) is ToleranceMode
        self._get_wrapped_object_().ToleranceUnit = value.value

    @property
    def mass_resolution(self) -> float:
        """
        Gets or sets the mass resolution for all scan arithmetic operations
        """
        return self._get_wrapped_object_().MassResolution

    @mass_resolution.setter
    def mass_resolution(self, value: float):
        """
        Gets or sets the mass resolution for all scan arithmetic operations
        """
        assert type(value) is float
        self._get_wrapped_object_().MassResolution = value

    @property
    def centroid_scan(self) -> CentroidStream:
        """
        Gets or sets A second data stream for the scan
        """
        return CentroidStream._get_wrapper_(self._get_wrapped_object_().CentroidScan)

    @centroid_scan.setter
    def centroid_scan(self, value: CentroidStream):
        """
        Gets or sets A second data stream for the scan
        """
        assert type(value) is CentroidStream
        self._get_wrapped_object_().CentroidScan = value._get_wrapped_object_()

    @property
    def always_merge_segments(self) -> bool:
        """
        Get or sets a value indicating whether scan + and - operators will merge data
        from scans which were not scanned over a similar range. Only applicable when
        scans only have a single segment. By default: Scans are considered incompatible
        if: The span of the scanned mass range differs by 10% The start or end of the
        scanned mass range differs by 10% If this is set as "true" then any mass ranges
        will be merged.
        """
        return self._get_wrapped_object_().AlwaysMergeSegments

    @always_merge_segments.setter
    def always_merge_segments(self, value: bool):
        """
        Get or sets a value indicating whether scan + and - operators will merge data
        from scans which were not scanned over a similar range. Only applicable when
        scans only have a single segment. By default: Scans are considered incompatible
        if: The span of the scanned mass range differs by 10% The start or end of the
        scanned mass range differs by 10% If this is set as "true" then any mass ranges
        will be merged.
        """
        assert type(value) is bool
        self._get_wrapped_object_().AlwaysMergeSegments = value

    @property
    def has_centroid_stream(self) -> bool:
        """
        Gets a value indicating whether this scan has a centroid stream.
        """
        return self._get_wrapped_object_().HasCentroidStream

    @property
    def centroid_stream_access(self) -> CentroidStream:
        """
        Gets A second data stream for the scan
        """
        return CentroidStream._get_wrapper_(self._get_wrapped_object_().CentroidStreamAccess)

    @property
    def has_noise_table(self) -> bool:
        """
        Gets a value indicating whether this scan has a noise table. This will be true
        only if the scan was constructed with the overload containing this table. Note
        that this is not related to having "noise and baseline" values with centroid
        stream data. This is a separate table, used for spectrum averaging and subtraction
        of orbitrap data
        """
        return self._get_wrapped_object_().HasNoiseTable

    @staticmethod
    def at_time(raw_file: RawFileAccess, time: float) -> Scan:
        """
        Create a scan object from a file and a retention time.
        
        Parameters:
        rawFile:
        File to read from
        
        time:
        time of Scan number to read
        
        Returns:
        The scan read, or null of the scan number if not valid
        """
        return Scan._get_wrapper_(Scan._wrapped_type.AtTime(raw_file._get_wrapped_object_(), time))

    @staticmethod
    def can_merged_scan( identical_flag: bool, current_scan: Scan, to_merge: Scan) -> bool:
        """
        test if 2 scans can be averaged or subtracted.
        
        Parameters:
        identicalFlag:
        Returned as "true" if all segments are the same
        
        currentScan:
        Current scan object
        
        toMerge:
        The scan to possibly add
        
        Returns:
        true if scans can be merged
        """
        return Scan._wrapped_type.CanMergeScan(identical_flag, current_scan._get_wrapped_object_(), to_merge._get_wrapped_object_())

    @staticmethod
    def create_scan_reader(cache_size: int) -> CachedScanProvider:
        """
        Create an object which can be used to read scans from a file, with optional caching.
        This is valuable if repeated operations (such as averaging) are expected over
        the same region of data. Scans returned from each call are unique objects, even
        if called repeatedly with the same scan number.
        
        Parameters:
        cacheSize:
        Number of scans cached. When set to 1 or more, this creates a FIFO, keeping track
        of the most recently read scans. If a scan in the FIFO is requested again, it
        is pulled from the cache. If a scan is not in the cache, then a new scan is read
        from the file. If the cache is full, the oldest scan is dropped. The newly read
        scan is that added to the FIFO cache. If size is set to 0, this makes a trivial
        object with no overheads, that directly gets scans from the file.
        
        Returns:
        Object to read scans from a file
        """
        return CachedScanProvider._get_wrapper_(Scan._wrapped_type.CreateScanReader(cache_size))

    @staticmethod
    def from_file(raw_file: RawFileAccess, scan_number: int) -> Scan:
        """
        Create a scan object from a file and a scan number.
        
        Parameters:
        rawFile:
        File to read from
        
        scanNumber:
        Scan number to read
        
        Returns:
        The scan read, or null of the scan number if not valid
        """
        assert type(scan_number) is int
        return Scan._get_wrapper_(Scan._wrapped_type.FromFile(raw_file._get_wrapped_object_(), scan_number))

    def to_centroid(current_scan: Scan) -> Scan:
        """
        Converts the segmented scan to centroid scan. Used to centroid profile data.
        
        Parameters:
        currentScan:
        The scan to centroid
        
        Returns:
        The centroided version of the scan
        """
        return Scan._get_wrapper_(Scan._wrapped_type.ToCentroid(current_scan._get_wrapped_object_()))

    def deep_clone(self) -> Scan:
        """
        Make a deep clone of this scan.
        
        Returns:
        An object containing all data in the input, and no shared references
        """
        return Scan._get_wrapper_(self._get_wrapped_object().DeepClone())

    def generate_frequency_table(self) -> List[float]:
        """
        generate frequency table for this scan. This method only applied to "FT" format
        scans which have mass to frequency calibration data. When a scan in constructed
        from processing algorithms, such as averaging, a frequency to mass converter
        is used to create this scan. This same converter can be used to create a frequency
        table, which would be needed when writing averaged (or subtracted) data to a
        raw file.
        
        Returns:
        The frequency table.
        """
        return self._get_wrapped_object().GenerateFrequencyTable()

    def generate_noise_table(self) -> List[NoiseAndBaseline]:
        """
        Generates a "noise and baseline table". This table is only relevant to FT format
        data. For other data, an empty list is returned. This table is intended for use
        when exporting processed (averaged, subtracted) scans to a raw file. If this
        scan is the result of a calculation such as "average of subtract" it may be constructed
        using an overload which includes a noise and baseline table. If so: that tale
        is returned. Otherwise, a table is generated by extracting data from the scan.
        
        Returns:
        The nose and baseline data
        """
        return [NoiseAndBaseline._get_wrapper_(n) for n in self._get_wrapped_object().GenerateNoiseTable()]

    def slice(self, mass_ranges: List[Range], trim_mass_range: bool, expand_profiles: bool) -> Scan:
        """
        Return a slice of a scan which only contains data within the supplied mass Range
        or ranges. For example: For a scan with data from m/z 200 to 700, and a single
        mass range of 300 to 400: This returns a new scan containing all data with the
        range 300 to 400.
        
        Parameters:
        massRanges:
        The mass ranges, where data should be retained. When multiple ranges are supplied,
        all data which is in at least one range is included in the returned scan
        
        trimMassRange:
        If this is true, then the scan will reset the scan's mass range to the bounds
        of the supplied mass ranges
        
        expandProfiles:
        This setting only applies when the scan has both profile and centroid data. If
        true: When there isa centroid near the start or end of a range, and the first
        or final "above zero" section of the profile includes that peak, then the profile
        is extended, to include the points which contribute to that peak. A maximum of
        10 points may be added
        
        Returns:
        A copy of the scan, with only the data in the supplied ranges
        """
        net_mass_ranges = [r._get_wrapped_object_() for r in mass_ranges]
        return Scan._get_wrapper_(self._get_wrapped_object().Slice(net_mass_ranges, trim_mass_range, expand_profiles))



from typing import List
from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data.business import BracketType


class SequenceInfo(NetWrapperBase):
    @property
    def column_width(self) -> List[int]:
        """
        Gets the display width of each sequence column
        """
        return self._get_wrapped_object_().ColumnWidth

    @property
    def type_to_column_position(self) -> List[int]:
        """
        Gets the column order (see home page?)
        """
        return self._get_wrapped_object_().TypeToColumnPosition

    @property
    def bracket(self) -> BracketType:
        """
        Gets the sequence bracket type. This determines which groups of samples use the
        same calibration curve.
        """
        return BracketType(self._get_wrapped_object_().Bracket)

    @property
    def user_private_label(self) -> List[str]:
        """
        Gets the set of column names for application specific columns
        """
        return self._get_wrapped_object_().UserPrivateLabel

    @property
    def tray_configuration(self) -> str:
        """
        Gets a description of the auto-sampler tray
        """
        return self._get_wrapped_object_().TrayConfiguration

    @property
    def user_label(self) -> List[str]:
        """
        Gets the user configurable column names
        """
        return self._get_wrapped_object_().UserLabel



from __future__ import annotations
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from typing import List
from fisher_py.data.business import Range, TraceType
from fisher_py.utils import to_net_list


class ChromatogramTraceSettings(NetWrapperBase):
    """
    Setting to define a chromatogram Trace.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings

    def __init__(self, *args):
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        elif len(args) == 1:
            arg = args[0]
            if type(arg) is ChromatogramTraceSettings:
                self._wrapped_object = self._wrapped_type(arg._get_wrapped_object_())
            elif type(arg) is TraceType:
                self._wrapped_object = self._wrapped_type(arg.value)
            else:
                raise ValueError('Unable to create chromatogram trace settings')
        elif len(args) == 2:
            filter_, range_ = args
            assert type(filter_) is str
            assert type(range_) is Range
            self._wrapped_object = self._wrapped_type(filter_, range_._get_wrapped_object_())

    @property
    def mass_range_count(self) -> int:
        """
        Gets or sets the number of mass ranges, or wavelength ranges for PDA.
        
        Value:
        Numeric count of mass ranges
        
        Remarks:
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        return self._get_wrapped_object_().MassRangeCount

    @mass_range_count.setter
    def mass_range_count(self, value: int):
        """
        Gets or sets the number of mass ranges, or wavelength ranges for PDA.
        
        Value:
        Numeric count of mass ranges
        
        Remarks:
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        assert type(value) is int
        self._get_wrapped_object_().MassRangeCount = value

    @property
    def include_reference(self) -> bool:
        """
        Gets or sets a value indicating whether reference and exception peaks are included
        in this chromatogram trace
        """
        return self._get_wrapped_object_().IncludeReference

    @include_reference.setter
    def include_reference(self, value: bool):
        """
        Gets or sets a value indicating whether reference and exception peaks are included
        in this chromatogram trace
        """
        assert type(value) is bool
        self._get_wrapped_object_().IncludeReference = value

    @property
    def fragment_mass(self) -> float:
        """
        Gets or sets the fragment mass for neutral fragment filters.
        
        Value:
        Floating point fragment mass for neutral fragment filters
        """
        return self._get_wrapped_object_().FragmentMass

    @fragment_mass.setter
    def fragment_mass(self, value: float):
        """
        Gets or sets the fragment mass for neutral fragment filters.
        
        Value:
        Floating point fragment mass for neutral fragment filters
        """
        assert type(value) is float
        self._get_wrapped_object_().FragmentMass = value

    @property
    def filter(self) -> str:
        """
        Gets or sets the filter used in searching scans during trace build
        """
        return self._get_wrapped_object_().Filter

    @filter.setter
    def filter(self, value: str):
        """
        Gets or sets the filter used in searching scans during trace build
        """
        assert type(value) is str
        self._get_wrapped_object_().Filter = value

    @property
    def delay_in_min(self) -> float:
        """
        Gets or sets the delay in minutes.
        
        Value:
        Floating point delay in minutes
        """
        return self._get_wrapped_object_().DelayInMin

    @delay_in_min.setter
    def delay_in_min(self, value: float):
        """
        Gets or sets the delay in minutes.
        
        Value:
        Floating point delay in minutes
        """
        assert type(value) is float
        self._get_wrapped_object_().DelayInMin = value

    @property
    def trace(self) -> TraceType:
        """
        Gets or sets the type of trace to construct
        
        Value:
        see ThermoFisher.CommonCore.Data.Business.TraceType for more details
        """
        return TraceType(self._get_wrapped_object_().Trace)

    @trace.setter
    def trace(self, value: TraceType):
        """
        Gets or sets the type of trace to construct
        
        Value:
        see ThermoFisher.CommonCore.Data.Business.TraceType for more details
        """
        assert type(value) is TraceType
        self._get_wrapped_object_().Trace = value.value

    @property
    def mass_ranges(self) -> List[Range]:
        """
        Gets or sets the mass ranges.
        
        Value:
        Array of mass ranges
        
        Remarks:
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        return [Range._get_wrapper_(r) for r in self._get_wrapped_object_().MassRanges]

    @mass_ranges.setter
    def mass_ranges(self, value: List[Range]):
        """
        Gets or sets the mass ranges.
        
        Value:
        Array of mass ranges
        
        Remarks:
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        assert type(value) is list
        #value = to_net_list([r._get_wrapped_object_() for r in value], Range._wrapped_type)
        value = [r._get_wrapped_object_() for r in value]
        self._get_wrapped_object_().MassRanges = value

    @property
    def compound_names(self) -> List[str]:
        """
        Gets or sets the compound names.
        """
        return self._get_wrapped_object_().CompoundNames

    @compound_names.setter
    def compound_names(self, value: List[str]):
        """
        Gets or sets the compound names.
        """
        assert type(value) is list
        value = to_net_list(value, str)
        self._get_wrapped_object_().CompoundNames = value

    def clone(self) -> ChromatogramTraceSettings:
        """
        Copies all of the items to from this object into the returned object.
        
        Returns:
        The clone.
        """
        return ChromatogramTraceSettings._get_wrapper_(self._get_wrapped_object().Clone())

    def get_mass_range(self, index: int) -> Range:
        """
        Gets a range value at 0-based index.
        
        Parameters:
        index:
        Index at which to retrieve the range
        
        Returns:
        ThermoFisher.CommonCore.Data.Business.Range value at give index
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.MassRangeCount
        to find out the count of mass ranges.
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        assert type(index) is int
        return Range._get_wrapper_(self._get_wrapped_object().GetMassRange(index))

    def set_mass_range(self, index: int, range: Range):
        """
        Sets a range value at 0-based index.
        
        Parameters:
        index:
        Index at which new range value is to be set
        
        range:
        New ThermoFisher.CommonCore.Data.Business.Range value to be set
        
        Remarks:
        Set count of mass ranges using ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.MassRangeCount
        before setting any mass ranges.
        If ThermoFisher.CommonCore.Data.Business.ChromatogramTraceSettings.Trace is MassRange
        then mass range values are used to build trace.
        """
        assert type(index) is int
        assert type(range) is Range
        self._get_wrapped_object().SetMassRange(index, range._get_wrapped_object_())




from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data.filter_enums import ActivationType


class Reaction(NetWrapperBase):
    """
    The Reaction interface. Defines a reaction for fragmenting an ion (an MS/MS stage).
    """

    def __init__(self, reaction_net):
        super().__init__()
        self._wrapped_object = reaction_net

    @property
    def precursor_mass(self) -> float:
        """
        Gets the precursor mass (mass acted on)
        """
        return self._get_wrapped_object_().PrecursorMass

    @property
    def collision_energy(self) -> float:
        """
        Gets the collision energy of this reaction
        """
        return self._get_wrapped_object_().CollisionEnergy

    @property
    def isolation_width(self) -> float:
        """
        Gets the isolation width of the precursor mass
        """
        return self._get_wrapped_object_().IsolationWidth

    @property
    def precursor_range_is_valid(self) -> bool:
        """
        Gets a value indicating whether precursor range is valid. If this is true, then
        ThermoFisher.CommonCore.Data.Business.IReaction.PrecursorMass is still the center
        of the range, but the values ThermoFisher.CommonCore.Data.Business.IReaction.FirstPrecursorMass
        and ThermoFisher.CommonCore.Data.Business.IReaction.LastPrecursorMass define
        the limits of the precursor mass range
        """
        return self._get_wrapped_object_().PrecursorRangeIsValid

    @property
    def first_precursor_mass(self) -> float:
        """
        Gets the start of the precursor mass range (only if ThermoFisher.CommonCore.Data.Business.IReaction.PrecursorRangeIsValid)
        """
        return self._get_wrapped_object_().FirstPrecursorMass

    @property
    def last_precursor_mass(self) -> float:
        """
        Gets the end of the precursor mass range (only if ThermoFisher.CommonCore.Data.Business.IReaction.PrecursorRangeIsValid)
        """
        return self._get_wrapped_object_().LastPrecursorMass

    @property
    def collision_energy_valid(self) -> bool:
        """
        Gets a value indicating whether collision energy is valid.
        """
        return self._get_wrapped_object_().CollisionEnergyValid

    @property
    def activation_type(self) -> ActivationType:
        """
        Gets the activation type.
        """
        return ActivationType(self._get_wrapped_object_().ActivationType)

    @property
    def multiple_activation(self) -> bool:
        """
        Gets a value indicating whether this is a multiple activation. In a table of
        reactions, a multiple activation is a second, or further, activation (fragmentation
        method) applied to the same precursor mass. Precursor mass values should be obtained
        from the original activation, and may not be returned by subsequent multiple
        activations.
        """
        return self._get_wrapped_object_().MultipleActivation

    @property
    def isolation_width_offset(self) -> float:
        """
        Gets the isolation width offset.
        """
        return self._get_wrapped_object_().IsolationWidthOffset




from fisher_py.net_wrapping import NetWrapperBase
from fisher_py.data.business import Range, Reaction
from fisher_py.data.filter_enums import (
    EventAccurateMass, FieldFreeRegionType, TriState, CompensationVoltageType, SectorScanType, ActivationType,
    PolarityType, MsOrderType, SourceFragmentationValueType, ScanModeType, ScanDataType, IonizationModeType,
    DetectorType, MassAnalyzerType, EnergyType
)


class ScanEvent(NetWrapperBase):
    """
    The ScanEvent interface. Determines how scans are done.
    """

    @property
    def field_free_region(self) -> FieldFreeRegionType:
        """
        Gets the field free region setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.FieldFreeRegionType for possible
        values
        """
        return FieldFreeRegionType(self._get_wrapped_object_().FieldFreeRegion)

    @property
    def enhanced(self) -> TriState:
        """
        Gets the enhanced scan setting.
        """
        return TriState(self._get_wrapped_object_().Enhanced)

    @property
    def multiple_photon_dissociation(self) -> TriState:
        """
        Gets the multi-photon dissociation setting.
        """
        return TriState(self._get_wrapped_object_().MultiplePhotonDissociation)

    @property
    def multiple_photon_dissociation_value(self) -> float:
        """
        Gets the multi-photon dissociation value.
        
        Value:
        Floating point multi-photon dissociation value
        """
        return self._get_wrapped_object_().MultiplePhotonDissociationValue

    @property
    def electron_capture_dissociation(self) -> TriState:
        """
        Gets the electron capture dissociation setting.
        """
        return TriState(self._get_wrapped_object_().ElectronCaptureDissociation)

    @property
    def electron_capture_dissociation_value(self) -> float:
        """
        Gets the electron capture dissociation value.
        
        Value:
        Floating point electron capture dissociation value
        """
        return self._get_wrapped_object_().ElectronCaptureDissociationValue

    @property
    def photo_ionization(self) -> TriState:
        """
        Gets the photo ionization setting.
        """
        return TriState(self._get_wrapped_object_().PhotoIonization)

    @property
    def pulsed_q_dissociation(self) -> TriState:
        """
        Gets pulsed dissociation setting.
        """
        return TriState(self._get_wrapped_object_().PulsedQDissociation)

    @property
    def pulsed_q_dissociation_value(self) -> float:
        """
        Gets the pulsed dissociation value.
        
        Value:
        Floating point pulsed dissociation value
        """
        return self._get_wrapped_object_().PulsedQDissociationValue

    @property
    def electron_transfer_dissociation(self) -> TriState:
        """
        Gets the electron transfer dissociation setting.
        """
        return TriState(self._get_wrapped_object_().ElectronTransferDissociation)

    @property
    def electron_transfer_dissociation_value(self) -> float:
        """
        Gets the electron transfer dissociation value.
        
        Value:
        Floating point electron transfer dissociation value
        """
        return self._get_wrapped_object_().ElectronTransferDissociationValue

    @property
    def higher_energy_ci_d(self) -> TriState:
        """
        Gets the higher energy CID setting.
        """
        return TriState(self._get_wrapped_object_().HigherEnergyCiD)

    @property
    def higher_energy_ci_d_value(self) -> float:
        """
        Gets the higher energy CID value.
        
        Value:
        Floating point higher energy CID value
        """
        return self._get_wrapped_object_().HigherEnergyCiDValue

    @property
    def ultra(self) -> TriState:
        """
        Gets the ultra scan setting.
        """
        return TriState(self._get_wrapped_object_().Ultra)

    @property
    def multiplex(self) -> TriState:
        """
        Gets the Multiplex type
        """
        return TriState(self._get_wrapped_object_().Multiplex)

    @property
    def param_b(self) -> TriState:
        """
        Gets the parameter b.
        """
        return TriState(self._get_wrapped_object_().ParamB)

    @property
    def param_f(self) -> TriState:
        """
        Gets the parameter f.
        """
        return TriState(self._get_wrapped_object_().ParamF)

    @property
    def multi_notch(self) -> TriState:
        """
        Gets the Multi notch (Synchronous Precursor Selection) type
        """
        return TriState(self._get_wrapped_object_().MultiNotch)

    @property
    def param_r(self) -> TriState:
        """
        Gets the parameter r.
        """
        return TriState(self._get_wrapped_object_().ParamR)

    @property
    def param_v(self) -> TriState:
        """
        Gets the parameter v.
        """
        return TriState(self._get_wrapped_object_().ParamV)

    @property
    def name(self) -> str:
        """
        Gets the event Name.
        """
        return self._get_wrapped_object_().Name

    @property
    def supplemental_activation(self) -> TriState:
        """
        Gets supplemental activation type setting.
        """
        return TriState(self._get_wrapped_object_().SupplementalActivation)

    @property
    def multi_state_activation(self) -> TriState:
        """
        Gets MultiStateActivation type setting.
        """
        return TriState(self._get_wrapped_object_().MultiStateActivation)

    @property
    def compensation_voltage(self) -> TriState:
        """
        Gets Compensation Voltage Option setting.
        """
        return TriState(self._get_wrapped_object_().CompensationVoltage)

    @property
    def compensation_volt_type(self) -> CompensationVoltageType:
        """
        Gets Compensation Voltage type setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.CompensationVoltageType for possible
        values
        """
        return CompensationVoltageType(self._get_wrapped_object_().CompensationVoltType)

    @property
    def scan_type_index(self) -> int:
        """
        Gets encoded form of segment and scan event number.
        
        Value:
        HIWORD == segment, LOWORD == scan type
        """
        return self._get_wrapped_object_().ScanTypeIndex

    @property
    def mass_count(self) -> int:
        """
        Gets number of (precursor) masses
        
        Value:
        The size of mass array
        """
        return self._get_wrapped_object_().MassCount

    @property
    def param_a(self) -> TriState:
        """
        Gets the parameter a.
        """
        return TriState(self._get_wrapped_object_().ParamA)

    @property
    def mass_range_count(self) -> int:
        """
        Gets the number of mass ranges for final scan
        
        Value:
        The size of mass range array
        """
        return self._get_wrapped_object_().MassRangeCount

    @property
    def source_fragmentation_info_count(self) -> int:
        """
        Gets the number of source fragmentation info values
        
        Value:
        The size of source fragmentation info array
        """
        return self._get_wrapped_object_().SourceFragmentationInfoCount

    @property
    def sector_scan(self) -> SectorScanType:
        """
        Gets the sector scan setting. Applies to 2 sector (Magnetic, electrostatic) Mass
        spectrometers, or hybrids.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SectorScanType for possible values
        """
        return SectorScanType(self._get_wrapped_object_().SectorScan)

    @property
    def lock(self) -> TriState:
        """
        Gets the lock scan setting.
        """
        return TriState(self._get_wrapped_object_().Lock)

    @property
    def polarity(self) -> PolarityType:
        """
        Gets the polarity of the scan.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.PolarityType for possible values
        """
        return PolarityType(self._get_wrapped_object_().Polarity)

    @property
    def ms_order(self) -> MsOrderType:
        """
        Gets the scan MS/MS power setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MSOrderType for possible values
        """
        return MsOrderType(self._get_wrapped_object_().MsOrder)

    @property
    def dependent(self) -> TriState:
        """
        Gets the dependent scan setting. A scan is "dependent" if the scanning method
        is based on analysis of data from a previous scan.
        """
        return TriState(self._get_wrapped_object_().Dependent)

    @property
    def source_fragmentation(self) -> TriState:
        """
        Gets source fragmentation scan setting.
        """
        return TriState(self._get_wrapped_object_().SourceFragmentation)

    @property
    def source_fragmentation_type(self) -> SourceFragmentationValueType:
        """
        Gets the source fragmentation type setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.SourceFragmentationValueType for
        possible values
        """
        return SourceFragmentationValueType(self._get_wrapped_object_().SourceFragmentationType)

    @property
    def scan_mode(self) -> ScanModeType:
        """
        Gets the scan type setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.ScanModeType for possible values
        """
        return ScanModeType(self._get_wrapped_object_().ScanMode)

    @property
    def scan_data(self) -> ScanDataType:
        """
        Gets the scan data format (profile or centroid).
        """
        return ScanDataType(self._get_wrapped_object_().ScanData)

    @property
    def ionization_mode(self) -> IonizationModeType:
        """
        Gets the ionization mode scan setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.IonizationModeType for possible
        values
        """
        return IonizationModeType(self._get_wrapped_object_().IonizationMode)

    @property
    def corona(self) -> TriState:
        """
        Gets the corona scan setting.
        """
        return TriState(self._get_wrapped_object_().Corona)

    @property
    def detector(self) -> DetectorType:
        """
        Gets the detector validity setting. The property ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.DetectorValue
        only contains valid information when this is set to "DetectorType.Valid"
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.DetectorType for possible values
        """
        return DetectorType(self._get_wrapped_object_().Detector)

    @property
    def detector_value(self) -> float:
        """
        Gets the detector value. This should only be used when valid. ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.Detector
        
        Value:
        Floating point detector value
        """
        return self._get_wrapped_object_().DetectorValue

    @property
    def wideband(self) -> TriState:
        """
        Gets the wideband scan setting.
        """
        return TriState(self._get_wrapped_object_().Wideband)

    @property
    def mass_analyzer(self) -> MassAnalyzerType:
        """
        Gets the mass analyzer scan setting.
        
        Value:
        See ThermoFisher.CommonCore.Data.FilterEnums.MassAnalyzerType for possible values
        """
        return MassAnalyzerType(self._get_wrapped_object_().MassAnalyzerType)

    @property
    def turbo_scan(self) -> TriState:
        """
        Gets the turbo scan setting.
        """
        return TriState(self._get_wrapped_object_().TurboScan)

    def get_activation(self, index: int) -> ActivationType:
        """
        Retrieves activation type at 0-based index.
        
        Parameters:
        index:
        Index of activation to be retrieved
        
        Returns:
        activation of MS step; See ThermoFisher.CommonCore.Data.FilterEnums.ActivationType
        for possible values
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassCount to get the
        count of activations.
        """
        return ActivationType(self._get_wrapped_object_().GetActivation(index))

    def get_energy(self, index: int) -> float:
        """
        Retrieves precursor(collision) energy value for MS step at 0-based index.
        
        Parameters:
        index:
        Index of precursor(collision) energy to be retrieved
        
        Returns:
        precursor(collision) energy of MS step
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassCount to get the
        count of energies.
        """
        return self._get_wrapped_object_().GetEnergy(index)

    def get_energy_valid(self, index: int) -> EnergyType:
        """
        Retrieves precursor(collision) energy validation flag at 0-based index.
        
        Parameters:
        index:
        Index of precursor(collision) energy validation to be retrieved
        
        Returns:
        precursor(collision) energy validation of MS step; See ThermoFisher.CommonCore.Data.FilterEnums.EnergyType
        for possible values
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassCount to get the
        count of precursor(collision) energy validations.
        """
        return EnergyType(self._get_wrapped_object_().GetEnergyValid(index))

    def get_first_precursor_mass(self, index: int) -> float:
        """
        Gets the first precursor mass. This is only valid data where "GetPrecursorRangeValidity"
        returns true for the same index.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The first mass
        """
        return self._get_wrapped_object_().GetFirstPrecursorMass(index)

    def get_is_multiple_activation(self, index: int) -> bool:
        """
        Retrieves multiple activations flag at 0-based index of masses.
        
        Parameters:
        index:
        Index of flag to be retrieved
        
        Returns:
        true if mass at given index has multiple activations; false otherwise
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassCount to get the
        count of masses.
        """
        return self._get_wrapped_object_().GetIsMultipleActivation(index)

    def get_isolation_width(self, index: int) -> float:
        """
        Gets the isolation width.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The isolation width
        """
        return self._get_wrapped_object_().GetIsolationWidth(index)

    def get_isolation_width_offset(self, index: int) -> float:
        """
        Gets the isolation width offset.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The isolation width offset
        """
        return self._get_wrapped_object_().GetIsolationWidthOffset(index)

    def get_last_precursor_mass(self, index: int) -> float:
        """
        Gets the last precursor mass. This is only valid data where "GetPrecursorRangeValidity"
        returns true for the same index.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The last mass
        """
        return self._get_wrapped_object_().GetLastPrecursorMass(index)

    def get_mass(self, index: int) -> float:
        """
        Retrieves mass value for MS step at 0-based index.
        
        Parameters:
        index:
        Index of mass value to be retrieved
        
        Returns:
        Mass value of MS step
        
        Exceptions:
        T:System.IndexOutOfRangeException:
        Will be thrown when index >= MassCount
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassCount to get the
        count of mass values.
        """
        return self._get_wrapped_object_().GetMass(index)

    def get_mass_range(self, index: int) -> Range:
        """
        Retrieves mass range for final scan at 0-based index.
        
        Parameters:
        index:
        Index of mass range to be retrieved
        
        Returns:
        Mass range for final scan at 0-based index
        
        Exceptions:
        T:System.IndexOutOfRangeException:
        Will be thrown when index >= MassRangeCount
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.MassRangeCount to
        get the count of mass ranges.
        """
        return Range._get_wrapper_(self._get_wrapped_object_().GetMassRange(index))

    def get_precursor_range_validity(self, index: int) -> bool:
        """
        Determine if a precursor range is valid.
        
        Parameters:
        index:
        The index.
        
        Returns:
        true if valid
        """
        return self._get_wrapped_object_().GetPrecursorRangeValidity(index)

    def get_reaction(self, index: int) -> Reaction:
        """
        Gets the reaction data for the mass at 0 based index. Descries how a particular
        MS/MS precursor mass is fragmented. Equivalent to calling GetMass, GetEnergy,
        GetPrecursorRangeValidity, GetFirstPrecursorMass GetLastPrecursorMass, GetIsolationWidth,
        GetIsolationWidthOffset, GetEnergyValid GetActivation, GetIsMultipleActivation.
        Depending on the implementation of the interface, this call may be more efficient
        than calling several of the methods listed.
        
        Parameters:
        index:
        index of reaction
        
        Returns:
        reaction details
        """
        return Reaction(self._get_wrapped_object_().Reactions[index])

    def get_source_fragmentation_info(self, index: int) -> float:
        """
        Retrieves a source fragmentation info value at 0-based index.
        
        Parameters:
        index:
        Index of source fragmentation info to be retrieved
        
        Returns:
        Source Fragmentation info value at 0-based index
        
        Exceptions:
        T:System.IndexOutOfRangeException:
        Will be thrown when index >= SourceFragmentationInfoCount
        
        Remarks:
        Use ThermoFisher.CommonCore.Data.Interfaces.IScanEventBase.SourceFragmentationInfoCount
        to get the count of source fragmentation info values.
        """
        return self._get_wrapped_object_().GetSourceFragmentationInfo(index)

    @property
    def accurate_mass(self) -> EventAccurateMass:
        """
        Gets the accurate mass setting.
        """
        return EventAccurateMass(self._get_wrapped_object_().AccurateMass)

    @property
    def is_valid(self) -> bool:
        """
        Gets a value indicating whether this event is valid.
        """
        return self._get_wrapped_object_().IsValid

    @property
    def is_custom(self) -> bool:
        """
        Gets a value indicating whether this is a custom event. A custom event implies
        that any scan derived from this event could be different. The scan type must
        be inspected to determine the scanning mode, and not the event.
        """
        return self._get_wrapped_object_().IsCustom

    @property
    def source_fragmentation_mass_range_count(self) -> int:
        """
        Gets the source fragmentation mass range count.
        """
        return self._get_wrapped_object_().SourceFragmentationMassRangeCount

    @property
    def mass_calibrator_count(self) -> int:
        """
        Gets the mass calibrator count.
        """
        return self._get_wrapped_object_().MassCalibratorCount

    def get_mass_calibrator(self, index: int) -> float:
        """
        Get the mass calibrator, at a given index.
        
        Parameters:
        index:
        The index, which should be from 0 to MassCalibratorCount -1
        
        Returns:
        The mass calibrator.
        
        Exceptions:
        T:System.IndexOutOfRangeException:
        Thrown when requesting calibrator above count
        """
        return self._get_wrapped_object_().GetMassCalibrator(index)

    def get_source_fragmentation_mass_range(self, index: int) -> Range:
        """
        Get the source fragmentation mass range, at a given index.
        
        Parameters:
        index:
        The index.
        
        Returns:
        The mass range.
        """
        return Range._get_wrapper_(self._get_wrapped_object_().GetSourceFragmentationMassRange(index))

    def __str__(self) -> str:
        """
        Convert to string.
        
        Returns:
        The converted scanning method.
        """
        return self._get_wrapped_object_().ToString()


import enum


class MsOrderType(enum.Enum):
    """
    Specifies scan power in scans.
    """
    Ng = -3
    Nl = -2
    Par = -1
    Any = 0
    Ms = 1
    Ms2 = 2
    Ms3 = 3
    Ms4 = 4
    Ms5 = 5
    Ms6 = 6
    Ms7 = 7
    Ms8 = 8
    Ms9 = 9
    Ms10 = 10



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher


class CommonCoreDataObject(NetWrapperBase):
    """
    CommonCoreData object is an abstract class. It includes a deep equals feature
    """

    def __init__(self):
        super().__init__()

    def deep_equals(self, value_to_compare: object) -> bool:
        """
        Provides a custom deep equality operation when checking for equality.
        
        Parameters:
        valueToCompare:
        The value to compare.
        
        Returns:
        True if the items are equal, false if they are not.
        """
        return self._get_wrapped_object().DeepEquals(value_to_compare._get_wrapped_object_())

    def equals(self, obj: object) -> bool:
        """
        Compares this object with another. Traverse the set of member variables to compare
        against the object that was passed in. Determines whether the specified System.Object
        is equal to the current System.Object.
        
        Parameters:
        obj:
        The System.Object to compare with the current System.Object.
        
        Returns:
        true if the specified System.Object is equal to the current System.Object; otherwise,
        false.
        
        Exceptions:
        T:System.NullReferenceException:
        The obj parameter is null.
        """
        return self._get_wrapped_object().Equals(obj._get_wrapped_object_())

    def get_hash_code(self) -> int:
        """
        Serves as a hash function for a particular type.
        
        Returns:
        A hash code for the current System.Object.
        """
        return self._get_wrapped_object().GetHashCode()

    def perform_default_settings(self):
        """
        Performs the default settings for the data object. This can overridden in each
        data object that implements the interface to perform initialization settings.
        """
        self._get_wrapped_object().PerformDefaultSettings()



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.filter_enums import MassAnalyzerType
from fisher_py.data.business import Scan
from fisher_py.raw_file_reader import RawFileAccess
from fisher_py.mass_precision_estimator import EstimatorResults
from fisher_py.utils import to_net_list
from typing import List


class PrecisionEstimate(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.MassPrecisionEstimator.PrecisionEstimate

    def __init__(self):
        super().__init__()
        self._wrapped_object = self._wrapped_type()
        self._raw_file = None

    @property
    def raw_file(self) -> RawFileAccess:
        """
        Gets the raw file as an open IRawData objects.
        """
        return self._raw_file

    @raw_file.setter
    def raw_file(self, value: RawFileAccess):
        """
        Sets the raw file as an open IRawData objects.
        """
        assert type(value) is RawFileAccess
        self._raw_file = value
        self._get_wrapped_object_().Rawfile = value._get_wrapped_object_()

    @property
    def scan_number(self) -> int:
        """
        Gets the scan number of the scan to be analyzed
        
        Value:
        The scan number.
        """
        return self._get_wrapped_object_().ScanNumber

    @scan_number.setter
    def scan_number(self, value: int):
        """
        Sets the scan number of the scan to be analyzed
        
        Value:
        The scan number.
        """
        assert type(value) is int
        self._get_wrapped_object_().ScanNumber = value

    def dispose(self):
        """
        Performs application-defined tasks associated with freeing, releasing, or resetting
        unmanaged resources.
        """
        self._get_wrapped_object_().Dispose()

    def get_ion_time(self, analyzer_type: MassAnalyzerType, scan: Scan, trailer_headings: List[str], trailer_values: List[str]) -> float:
        """
        Calculate the ion time (fill (traps and FT) or dwell time (quads))
        
        Parameters:
        analyzerType:
        The analyzer type
        
        scan:
        The scan to process
        
        trailerHeadings:
        The trailer extra data headings
        
        trailerValues:
        The trailer extra data values
        
        Returns:
        The calculated ion time
        """
        assert type(analyzer_type) is MassAnalyzerType
        assert type(scan) is Scan
        assert type(trailer_headings) is list
        assert type(trailer_values) is list
        trailer_headings = to_net_list(trailer_headings, str)
        trailer_values = to_net_list(trailer_values, str)
        return self._get_wrapped_object_().GetIonTime(analyzer_type.value, scan._get_wrapped_object_(), trailer_headings, trailer_values)

    def get_mass_precision_estimate(self, scan: Scan, analyzer_type: MassAnalyzerType, ion_time: float, resolution: float) -> List[EstimatorResults]:
        """
        Gets mass precision estimate and stores them in a class property list of classes
        This method will throw an Exception or ArgumentException if a problem occurs
        during processing.
        
        Parameters:
        scan:
        The scan to process
        
        analyzerType:
        The analyzer type for the scan
        
        ionTime:
        The ion time for the scan
        
        resolution:
        The resolution for the scan
        
        Returns:
        Returns the list of Mass Precision Estimation results
        """
        return self._get_wrapped_object_().GetMassPrecisionEstimate(scan, analyzer_type, ion_time, resolution)

    def get_mass_precision_estimate(self) -> List[EstimatorResults]:
        """
        Gets mass precision estimate and stores them in a class property list of classes
        This method will throw an Exception or ArgumentException if a problem occurs
        during processing.
        
        Returns:
        Returns the list of Mass Precision Estimation results
        """
        return [EstimatorResults._get_wrapper_(e) for e in self._get_wrapped_object_().GetMassPrecisionEstimate()]



from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import GenericDataTypes


class HeaderItem(NetWrapperBase):
    """
    Defines the format of a log entry, including label (name of the field), data
    type, and numeric formatting.
    """

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.HeaderItem

    def __init__(self, *args):
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        else:
            label, data_type, string_length_or_precision = args[0], args[1], args[2]
            assert type(label) is str
            assert type(data_type) is GenericDataTypes
            assert type(string_length_or_precision) is int

            if len(args) == 3:
                is_scientific_notation = False
            elif len(args) == 4:
                is_scientific_notation = args[3]
                assert type(is_scientific_notation) is bool
            else:
                raise ValueError('Unable to create header item.')

            self._wrapped_object = self._wrapped_type(label, data_type.value, string_length_or_precision, is_scientific_notation)

    @property
    def label(self) -> str:
        """
        Gets or sets the display label for the field. For example: If this a temperature,
        this label may be "Temperature" and the DataType may be "GenericDataTypes.FLOAT"
        """
        return self._get_wrapped_object_().Label

    @label.setter
    def label(self, value: str):
        """
        Gets or sets the display label for the field. For example: If this a temperature,
        this label may be "Temperature" and the DataType may be "GenericDataTypes.FLOAT"
        """
        assert type(value) is str
        self._get_wrapped_object_().Label = value

    @property
    def data_type(self) -> GenericDataTypes:
        """
        Gets or sets the data type for the field
        """
        return GenericDataTypes(self._get_wrapped_object_().DataType)

    @data_type.setter
    def data_type(self, value: GenericDataTypes):
        """
        Gets or sets the data type for the field
        """
        assert type(value) is GenericDataTypes
        self._get_wrapped_object_().DataType = value.value

    @property
    def string_length_or_precision(self) -> int:
        """
        Gets or sets the precision, if the data type is float or double, or string length
        of string fields.
        """
        return self._get_wrapped_object_().StringLengthOrPrecision

    @string_length_or_precision.setter
    def string_length_or_precision(self, value: int):
        """
        Gets or sets the precision, if the data type is float or double, or string length
        of string fields.
        """
        assert type(value) is int
        self._get_wrapped_object_().StringLengthOrPrecision = value

    @property
    def is_scientific_notation(self) -> bool:
        """
        Gets or sets a value indicating whether a number should be displayed in scientific
        notation
        """
        return self._get_wrapped_object_().IsScientificNotation

    @is_scientific_notation.setter
    def is_scientific_notation(self, value: bool):
        """
        Gets or sets a value indicating whether a number should be displayed in scientific
        notation
        """
        assert type(value) is bool
        self._get_wrapped_object_().IsScientificNotation = value

    @property
    def is_numeric(self) -> bool:
        """
        Gets a value indicating whether this is considered numeric data. This is the
        same test as performed for StatusLogPlottableData". Integer types: short and
        long (signed and unsigned) and floating types: float and double are defined as
        numeric.
        """
        return self._get_wrapped_object_().IsNumeric

    def format_value(self, value: str) -> str:
        """
        Re-formats the specified value per the current header's settings.
        
        Parameters:
        value:
        The value, as a string.
        
        Returns:
        The formatted value.
        """
        return self._get_wrapped_object().FormatValue(value)

    def is_variable_header(self, fields: int) -> bool:
        """
        Tests whether this is a variable header. A "variable header", if present as the
        first field in a table of headers, defines that each record has a variable number
        of valid fields. The first field in each data record will then be converted to
        "validity flags" which determine which of the fields in a data record have valid
        values.
        
        Parameters:
        fields:
        The number of fields in the header.
        
        Returns:
        True if this specifies that "variable length" records are used.
        """
        return self._get_wrapped_object().IsVariableHeader(fields)





from typing import Tuple, List
from fisher_py.raw_file_reader import RawFileReaderAdapter
from fisher_py.data.filter_enums import MsOrderType, MassAnalyzerType
from fisher_py.data.business import TraceType, ChromatogramTraceSettings, Range, MassOptions
from fisher_py.data import ToleranceUnits, FtAverageOptions, Device
import numpy as np


class RawFile(object):
    """
    Allows to access *.RAW files used by ThermoFisher to store MS measurements.
    NOTE: This class only provides limited access to all the functionalities and can serve as 
    an example how to use the module wihtin a project. For full access use RawFileReaderAdapter.file_factory()
    """

    @property
    def path(self) -> str:
        """
        Path of the raw file
        """
        return self._raw_file_access.path

    @property
    def number_of_scans(self) -> int:
        """
        Number of scans / spectra
        """
        return self._raw_file_access.run_header_ex.spectra_count

    @property
    def first_scan(self) -> int:
        """
        First scan number
        """
        return self._raw_file_access.run_header.first_spectrum

    @property
    def last_scan(self) -> int:
        """
        Last scan number
        """
        return self._raw_file_access.run_header.last_spectrum

    @property
    def total_time_min(self) -> float:
        """
        Total time of experiment in minutes
        """
        return self._raw_file_access.run_header.end_time

    @property
    def ms2_filter_masses(self) -> np.ndarray:
        """
        Available masses for MS2 filtering (precursor masses)
        """
        return self._ms2_filter_unique_filter_masses

    def __init__(self, path: str):
        self._path = path
        self._raw_file_access = RawFileReaderAdapter.file_factory(path)
        self._raw_file_access.select_instrument(Device.MS, 1)

        # fetch retention times and scan numbers
        scan_numbers, rt = self._get_scan_numbers_and_retention_times_()
        self._scan_numbers = scan_numbers
        self._retention_times = rt
        self._spectrum_cache = dict()
        self._result_string_cache = dict()

        # fetch retention times and scan numbers for MS1 only
        scan_numbers, rt = self._get_ms_scan_numbers_and_retention_times_(MsOrderType.Ms)
        self._ms1_scan_numbers = scan_numbers
        self._ms1_retention_times = rt

        # fetch retention times and scan numbers for MS2 only
        scan_numbers, rt = self._get_ms_scan_numbers_and_retention_times_(MsOrderType.Ms2)
        self._ms2_scan_numbers = scan_numbers
        self._ms2_retention_times = rt

        # fetch filters for MS2
        scan_numbers, filter_masses = self._get_ms2_scan_numbers_and_masses_()
        self._ms2_filter_scan_numbers = scan_numbers
        self._ms2_filter_masses = filter_masses
        self._ms2_filter_unique_filter_masses = np.array(sorted(list(set(self._ms2_filter_masses))))

    def _get_scan_numbers_and_retention_times_(self) -> Tuple[List[int], List[float]]:
        first_scan = self.first_scan
        scan_numbers = list()
        retention_times = list()

        for i in range(self.number_of_scans):
            scan_number = i + first_scan
            rt = self._raw_file_access.retention_time_from_scan_number(scan_number)
            scan_numbers.append(scan_number)
            retention_times.append(rt)

        return scan_numbers, retention_times

    def _get_ms2_scan_numbers_and_masses_(self) -> Tuple[List[float], List[float]]:
        first_scan = self.first_scan
        scan_numbers = list()
        filter_mass_values = list()
        scan_events = self._raw_file_access.get_scan_events(self.first_scan, self.last_scan)

        for i, scan_event in enumerate(scan_events):
            if scan_event.ms_order != MsOrderType.Ms2:
                continue
            scan_number = i + first_scan
            precursor_mass = self._get_scan_filter_precursor_mass_(scan_number)
            if precursor_mass is None:
                continue

            scan_numbers.append(scan_number)
            filter_mass_values.append(precursor_mass)

        return scan_numbers, filter_mass_values


    def _get_ms_scan_numbers_and_retention_times_(self, ms_order: MsOrderType) -> Tuple[List[int], List[float]]:
        first_scan = self.first_scan
        scan_numbers = list()
        retention_times = list()
        scan_events = self._raw_file_access.get_scan_events(self.first_scan, self.last_scan)
        
        for i, scan_event in enumerate(scan_events):
            if scan_event.ms_order != ms_order:
                continue
            scan_number = i + first_scan
            scan_numbers.append(scan_number)
            retention_times.append(self._raw_file_access.retention_time_from_scan_number(scan_number))

        return np.array(scan_numbers), np.array(retention_times)

    def _get_scan_filter_precursor_mass_(self, scan_number: int) -> float:
        scan_event = self._raw_file_access.get_scan_event_for_scan_number(scan_number)
        if scan_event.ms_order != MsOrderType.Ms2:
            return None
        return scan_event.get_reaction(0).precursor_mass

    def _get_scan_(self, scan_number: int) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
        mass_analyzer = self._raw_file_access.get_scan_event_for_scan_number(scan_number).mass_analyzer

        if mass_analyzer == MassAnalyzerType.MassAnalyzerFTMS:
            spectrum = self._raw_file_access.get_centroid_stream(scan_number, False)
            positions = np.array(spectrum.masses)
            intensities = np.array(spectrum.intensities)
            charges = np.array(spectrum.charges)
        else:
            spectrum = self._raw_file_access.get_segmented_scan_from_scan_number(scan_number, None)
            positions = np.array(spectrum.masses)
            intensities = np.array(spectrum.intensities)
            charges = np.zeros(positions.shape)
        return positions, intensities, charges

    def get_chromatogram(self, mz: float, tolerance: float, trace_type: TraceType=TraceType.MassRange, tolerance_units: ToleranceUnits=ToleranceUnits.ppm, ms_filter: str='ms') -> Tuple[np.ndarray, np.ndarray]:
        """
        Gets chromatogram
        :param mz: Mass/Charge value for mass range chromatogram
        :param tolerace: Tolerance for mass range chromatogram
        :param tolerance_units: Units of the mass tolerance (ppm by default)
        :param ms_filter: Type of MS data (ms or ms2)
        :param trace_type: Type of chromatogram (BasePeek, TIC (total ion current), MassRange (XIC))

        :return: array containing retention times and array containing intensity values
        """

        trace_settings = ChromatogramTraceSettings(trace_type)
        trace_settings.filter = ms_filter
        tolerance_arg = None

        if trace_type == TraceType.MassRange:
            trace_settings.mass_ranges = [Range(mz, mz)]
            tolerance_arg = MassOptions(tolerance, tolerance_units)

        chromatogram_raw = self._raw_file_access.get_chromatogram_data([trace_settings], -1, -1, tolerance_arg)
        return np.array(chromatogram_raw.positions_array[0]), np.array(chromatogram_raw.intensities_array[0])

    def get_tic_ms2(self, precursor_mz: float, tolerance: float=10e-3) -> Tuple[np.ndarray, np.ndarray]:
        """
        Get total ion current in MS2 for a given precursor mass.
        NOTE: This method does not yet support all mass tolerance units
        :param precursor_mz: Precursor mass
        :param tolerance: Mass tolerance (in ppm)
        :returns: Tuple of (retention_times, total_ion_current_intensities)
        """
        tic_rt, tic_intensities = list(), list()
        for n in range(self.first_scan, self.last_scan+1):
            scan_event = self._raw_file_access.get_scan_event_for_scan_number(n)
            precursor_mass = self._get_scan_filter_precursor_mass_(n)

            # skip scan if not MS2 or the precursor mass does not match the filter criteria
            if scan_event.ms_order != MsOrderType.Ms2 or precursor_mass is None or abs(precursor_mass - precursor_mz) > tolerance:
                continue

            _, y, _ = self._get_scan_(n)
            tic_rt.append(self._raw_file_access.retention_time_from_scan_number(n))
            tic_intensities.append(np.sum(y))

        return np.array(tic_rt), np.array(tic_intensities)

    def get_scan_ms1(self, rt: float) -> Tuple[np.ndarray, np.ndarray, np.ndarray, float]:
        """
        Gets MS1 (MS) spectrum for a given retention time value in minutes
        :param rt: retention time value in minutes

        :returns: Three arrays containing Mass/Charge values, intensity values and charge values as well as the actual retention time
        """
        scan_number, found_rt = self.get_ms1_scan_number_from_retention_time(rt)
        rts, intensities, charges, _ = self.get_scan_from_scan_number(scan_number)
        return rts, intensities, charges, found_rt

    def get_scan_ms2(self, rt: float, precursor_mz: float=None) -> Tuple[np.ndarray, np.ndarray, float]:
        """
        Gets MS2 spectrum for a given retention time value in minutes
        :param rt: retention time value in minutes
        :param recursor_mz: Optional precursor mass value to filter spectra

        :returns: Three arrays containing Mass/Charge values, intensity values and charge values as well as the actual retention time
        """
        scan_number, found_rt = self.get_ms2_scan_number_from_retention_time(rt, precursor_mz)
        rts, intensities, charges, _ = self.get_scan_from_scan_number(scan_number)
        return rts, intensities, charges, found_rt

    def get_scan(self, rt: float) -> Tuple[np.ndarray, np.ndarray, np.ndarray, str]:
        return self.get_scan_from_scan_number(self.get_scan_number_from_retention_time(rt))


    def get_average_ms2_scans_by_rt(self, rt_from: float, rt_to: float, precursor_mass: float, tolerance: float=10, tolerance_units: ToleranceUnits=ToleranceUnits.ppm) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
        """
        Averages MS spectra over a given range using a precursor filter
        :param rt_from: Retention time start in minutes
        :param rt_to: Retention time end in minutes
        :param precursor_mass: Precursor mass for filtering
        :param tolerance: Mass tolerance for binning
        :param tolerance_units: Mass tolerance units
        :returns: Three arrays containing mass/charge, intensities and charges
        """
        start_scan, _ = self.get_ms2_scan_number_from_retention_time(rt_from)
        end_scan, _ = self.get_ms2_scan_number_from_retention_time(rt_to)
        return self.get_averaged_ms2_scans(start_scan, end_scan, precursor_mass, tolerance, tolerance_units)

    def get_averaged_ms2_scans(self, start_scan: int, end_scan: int, precursor_mass: float, tolerance: float=10, tolerance_units: ToleranceUnits=ToleranceUnits.ppm) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
        """
        Averages MS spectra over a given range using a precursor filter
        :param start_scan: Number of start scan
        :param end_scan: Number of end scan
        :param precursor_mass: Precursor mass for filtering
        :param tolerance: Mass tolerance for binning
        :param tolerance_units: Mass tolerance units
        :returns: Three arrays containing mass/charge, intensities and charges
        """
        
        rounded_precursor = round(precursor_mass, 4)
        template_string = self._raw_file_access.get_scan_event_string_for_scan_number(start_scan)
        filter_string = f'FTMS + p ESI d Full ms2 {rounded_precursor}@{template_string.split("@")[1]}'
        mass_options = MassOptions(tolerance, tolerance_units, 4)
        average_options = FtAverageOptions()

        averaged_scans = self._raw_file_access.average_scans_in_scan_range(start_scan, end_scan, filter_string, mass_options, average_options)
        masses = np.array(averaged_scans.preferred_masses)
        intensities = np.array(averaged_scans.preferred_intensities)
        charges = averaged_scans.centroid_scan.charges if averaged_scans.has_centroid_stream else np.zeros(masses.shape)

        return masses, intensities, charges

    def get_scan_from_scan_number(self, scan_number: int) -> Tuple[np.ndarray, np.ndarray, np.ndarray, str]:
        """
        Get scan data from a scan number. The data returned is structured in a tuple as follows:
            (masses, intensities, ion_charges, scan_event_descriptions)

        :param scan_number: The number of the scan
        :returns: Tuple organized as (masses, intensities, ion_charges, scan_event_descriptions)
        """
        if scan_number < self.first_scan or scan_number > self.last_scan:
            raise ValueError(f'The scan number {scan_number} is out of bounds. Valid range {self.first_scan} - {self.last_scan}.')

        if scan_number in self._spectrum_cache:
            return self._spectrum_cache[scan_number][0], self._spectrum_cache[scan_number][1], self._spectrum_cache[scan_number][2], self._result_string_cache[scan_number]

        positions, intensities, charges = self._get_scan_(scan_number)
        scan_event_str = self._raw_file_access.get_scan_event_string_for_scan_number(scan_number)
        return positions, intensities, charges, scan_event_str

    def get_retention_time_from_scan_number(self, scan_number: int) -> float:
        """
        Get the retention time (in minutes) from a scan number
        :param scan_number: The number of the scan
        :returns: Retention time in minutes
        """
        if scan_number < self.first_scan or scan_number > self.last_scan:
            raise ValueError(f'The scan number {scan_number} is out of bounds. Valid range {self.first_scan} - {self.last_scan}.')
        
        idx = np.argmin(np.abs(self._scan_numbers - scan_number))
        return self._retention_times[idx]

    def get_scan_number_from_retention_time(self, rt: float) -> int:
        """
        Get the closest scan number from a given retention time (in minutes). If the retention time is smaller than zero
        this will always return the first scan number and if it is larger than the latest entry it will return the biggest
        scan number.
        :param rt: Retention time (in minutes)
        :returns: Scan number
        """
        if rt < 0 or rt > self.total_time_min:
            raise ValueError(f'The retiontion time {rt} is out of bounds. Valid range 0 - {self.total_time_min}.')

        idx = np.argmin(np.abs(self._retention_times - rt))
        return int(self._scan_numbers[idx])

    def get_ms1_scan_number_from_retention_time(self, rt: float) -> Tuple[int, float]:
        """
        Get the closest scan number in MS1 spectra from a given retention time (in minutes). If the retention time is smaller than zero
        this will always return the first scan number and if it is larger than the latest entry it will return the biggest
        scan number.
        :param rt: Retention time (in minutes)
        :returns: Scan number
        """
        if rt < 0 or rt > self.total_time_min:
            raise ValueError(f'The retiontion time {rt} is out of bounds. Valid range 0 - {self.total_time_min}.')

        idx = np.argmin(np.abs(self._ms1_retention_times - rt))
        found_rt = self._ms1_retention_times[idx]
        found_scan_nr = self._ms1_scan_numbers[idx]
        return int(found_scan_nr), found_rt

    def get_ms2_scan_number_from_retention_time(self, rt: float, precursor_mz: float=None, tolerance_ppm = 10e-3) -> Tuple[int, float]:
        """
        Get the closest scan number in MS2 spectra from a given retention time (in minutes). If the retention time is smaller than zero
        this will always return the first scan number and if it is larger than the latest entry it will return the biggest
        scan number.
        NOTE: This method does not yet support all mass tolerance units
        :param rt: Retention time (in minutes)
        :param precursor_mz: Precursor mass
        :param tolerance: Mass tolerance to precursor (in ppm)
        :returns: Scan number
        """
        if rt < 0 or rt > self.total_time_min:
            raise ValueError(f'The retiontion time {rt} is out of bounds. Valid range 0 - {self.total_time_min}.')

        if precursor_mz is None:
            idx = np.argmin(np.abs(self._ms2_retention_times - rt))
            found_rt = self._ms2_retention_times[idx]
            found_scan_nr = int(self._ms2_scan_numbers[idx])
            return found_scan_nr, found_rt
        else:
            ms2_idxs = np.abs(self._ms2_filter_masses - precursor_mz) <= tolerance_ppm
            ms2_scans = self._ms2_filter_scan_numbers[ms2_idxs]
            ms2_rts = self._retention_times[ms2_scans.astype(int) + self.first_scan]
            idx = np.argmin(np.abs(ms2_rts - rt))
            found_rt = ms2_rts[idx]
            found_scan_nr = int(ms2_scans[idx])
            return found_scan_nr, found_rt

    def get_scan_event_str_from_scan_number(self, scan_number: int) -> str:
        """
        Get the scan event description text from a scan number.
        :param scan_number: Scan number to retrieve the description from
        :returns: Scan event description
        """
        if scan_number < self.first_scan or scan_number > self.last_scan:
            raise ValueError(f'The scan number {scan_number} is out of bounds. Valid range {self.first_scan} - {self.last_scan}.')
        
        if scan_number in self._result_string_cache:
            return self._result_string_cache[scan_number]

        _, _, _, event_str = self.get_scan_from_scan_number(scan_number)
        return event_str



from fisher_py.raw_file_reader.scan_dependents import ScanDependents
from fisher_py.raw_file_reader.raw_file_access import RawFileAccess
from fisher_py.raw_file_reader.raw_file_reader_adapter import RawFileReaderAdapter



from __future__ import annotations
from typing import Any
from fisher_py.net_wrapping import NetWrapperBase, ThermoFisher
from fisher_py.data.business import MassOptions
from fisher_py.utils import is_number


class Range(NetWrapperBase):

    _wrapped_type = ThermoFisher.CommonCore.Data.Business.Range

    def __init__(self, *args):
        super().__init__()

        if len(args) == 0:
            self._wrapped_object = self._wrapped_type()
        elif len(args) == 1:
            from_ = args[0]
            assert type(from_) is Range
            self._wrapped_object = self._wrapped_type(from_._wrapped_object)
        elif len(args) == 2:
            arg1, arg2 = args
            if is_number(arg1) and is_number(arg2):
                self._wrapped_object = self._wrapped_type(float(arg1), float(arg2))
            elif type(arg1) is Range and is_number(arg2):
                self._wrapped_object = self._wrapped_type(arg1._wrapped_object, float(arg2))
            elif type(arg1) is Range and type(arg2) is MassOptions:
                self._wrapped_object = self._wrapped_type(arg1._wrapped_object, arg2._wrapped_object)
    
    @property
    def low(self) -> float:
        """
        Gets or sets the low end of range
        """
        return self._get_wrapped_object_().Low

    @low.setter
    def low(self, value: float):
        """
        Gets or sets the low end of range
        """
        assert type(value) is float
        self._get_wrapped_object_().Low = value

    @property
    def high(self) -> float:
        """
        Gets or sets the high end of range
        """
        return self._get_wrapped_object_().High

    @high.setter
    def high(self, value: float):
        """
        Gets or sets the high end of range
        """
        assert type(value) is float
        self._get_wrapped_object_().High = value

    @staticmethod
    def create(low: float, high: float) -> Range:
        """
        Create an immutable (constant) range from low and high.
        
        Parameters:
        low:
        The low.
        
        high:
        The high.
        
        Returns:
        The range.
        """
        assert is_number(low)
        assert is_number(high)
        return Range._get_wrapper_(Range._wrapped_type.Create(float(low), float(high)))

    def create_from_cetner_and_delta(center: float, delta: float) -> Range:
        """
        Create an immutable (constant) range from center and delta, such that the range
        is center +/- delta.
        
        Parameters:
        center:
        The center.
        
        delta:
        The delta.
        
        Returns:
        The ThermoFisher.CommonCore.Data.Business.Range.
        """
        assert is_number(center)
        assert is_number(delta)
        return Range._get_wrapper_(Range._wrapped_type.CreateFromCenterAndDelta(float(center), float(delta)))

    def compare_to(self, other: Range) -> int:
        """
        Compares the current object with another object of the same type.
        
        Parameters:
        other:
        An object to compare with this object.
        
        Returns:
        A 32-bit signed integer that indicates the relative order of the objects being
        compared. The return value has the following meanings: Value Meaning Less than
        zero This object is less than the other parameter. Zero This object is equal
        to other. Greater than zero This object is greater than other.
        """
        return self._get_wrapped_object().CompareTo(other._get_wrapped_object_())

    def equals(self, obj: object) -> bool:
        """
        Indicates whether this instance and a specified object are equal.
        
        Parameters:
        obj:
        Another object to compare to.
        
        Returns:
        true if obj and this instance are the same type and represent the same value;
        otherwise, false.
        """
        return self._get_wrapped_object().Equals(obj)

    def get_hash_code(self) -> int:
        """
        Returns the hash code for this instance.
        
        Returns:
        A 32-bit signed integer that is the hash code for this instance.
        """
        return self._get_wrapped_object().GetHashCode()

    def includes(self, value: float) -> bool:
        """
        Test for inclusion.
        
        Parameters:
        value:
        The value.
        
        Returns:
        True if in range
        """
        assert is_number(value)
        return self._get_wrapped_object().Includes(float(value))

    def __eq__(self, o: Range) -> bool:
        return self._get_wrapped_object_() == o._get_wrapped_object_()

    def __ne__(self, o: Range) -> bool:
        return not self == o
